from baseProblem import MatrixProblem
import numpy as np

class TSP(MatrixProblem):
    _optionalData = []
    probType = 'TSP'
    expectedArgs = ['graph', 'objective']
    allowedGoals = ['min', 'max', 'minimum', 'maximum']
    showGoal = True
    allowRevisit = False # True is unimplemented properly yet
#    _init = False
    start = None 
   
    returnToStart = True
    
#    def __setattr__(self, attr, val): 
#        if self._immutable: self.err('openopt TSP instances are immutable, arguments should pass to constructor or solve()')
#        self.__dict__[attr] = val
    
    def __init__(self, *args, **kw):
        self.goal = 'min'
        self.objective = 'weight'
        MatrixProblem.__init__(self, *args, **kw)
        self.__init_kwargs = kw
        self. _immutable = True
        
    def manage(self, *args, **kw):
        kw['routine'] = 'manage'
        r = self.solve(*args, **kw)
        return r
        
    def solve(self, *args, **kw):
        import FuncDesigner as fd, openopt as oo 
        
        if len(args) > 1:
            self.err('''
            incorrect number of arguments for solve(), 
            must be at least 1 (solver), other must be keyword arguments''')
        if self.start is None and not self.returnToStart:
            self.err('for returnToStart=False mode you should provide start, other cases are unimplemented yet')
        solver = args[0] if len(args) != 0 else kw.get('solver', self.solver)
        routine = kw.pop('routine', 'solve')
        KW = self.__init_kwargs.copy()
        KW.update(kw)
        
        objective = KW.get('objective', self.objective)
        if isinstance(objective, (list, tuple, set)):
            nCriteria = len(self.objective)
            if 3 * nCriteria != np.asarray(self.objective).size:
                objective = [(objective[3*i], objective[3*i+1], objective[3*i+2]) for i in range(int(round(np.asarray(self.objective).size / 3)))]
            if len(objective) == 1:
                KW['fTol'], KW['goal'] = objective[0][1:]
        else:
            objective = [(self.objective, KW.get('fTol', getattr(self, 'fTol')), KW.get('goal', getattr(self, 'goal')))]

        nCriteria = len(objective)
        isMOP = nCriteria > 1
        mainCr = objective[0][0]
         
        
        solverName = solver if type(solver) == str else solver.__name__
        is_interalg = solverName == 'interalg'
        is_glp = solverName == 'sa'
        if is_glp:
            assert nCriteria == 1, 'you cannot solve multiobjective tsp by the solver'
            
        is_interalg_raw_mode = is_interalg and KW.get('dataHandling', oo.oosolver(solver).dataHandling) in ('auto','raw')
        KW.pop('objective', None)
        P = oo.MOP if nCriteria > 1 else oo.GLP if is_interalg else oo.MILP if not is_glp else oo.GLP

        import networkx as nx
        graph = self.graph # must be networkx Graph instance
        
        init_graph_is_directed = graph.is_directed()
        init_graph_is_multigraph = graph.is_multigraph()
        if not init_graph_is_multigraph or not init_graph_is_directed:
            graph = nx.MultiDiGraph(graph) #if init_graph_is_directed else nx.MultiGraph(graph)
        
        nodes = graph.nodes()
        edges = graph.edges()
        n = len(nodes)
        m = len(edges)

        node2index = dict([(node, i) for i, node in enumerate(nodes)])
        
        # TODO: implement MOP with interalg_gdp mode (requires interpolation interval analysis for non-monotone funcs)
        interalg_gdp = 1
        if not is_interalg:# or isMOP:
            # !!!!!!!!!!!!! TODO: add handling of MOP with interalg_gdp?
            interalg_gdp = 0 
        
        edge_ind2x_ind_val = {}#used by interalg only, but present in encode() args
        if interalg_gdp:
            x = []
        else:
            pass
            #x = fd.oovars(m, domain=bool)

        #cr_values = dict([(obj[0], []) for obj in objective])
        cr_values = {}
        constraints = []
        EdgesDescriptors, EdgesCoords = [], []
        # mb rework it by successors etc?
        
        Funcs = {}
        Cons = KW.pop('constraints', [])
        if type(Cons) not in (list, tuple):
            Cons = [Cons]
        usedValues = getUsedValues(Cons)
        usedValues.update(getUsedValues([obj[0] for obj in objective]))
        
        MainCr = mainCr if type(mainCr) in (str, np.str_) else list(usedValues)[0]
        
        isMainCrMin = objective[0][2] in ('min', 'minimum')
        node_out_edges_num = []
        for node in nodes:
            Edges = graph[node]
            node_out_edges_num.append(len(Edges))
            out_nodes = Edges.keys()
            if len(out_nodes) == 0:
                self.err('input graph has node %s that does not lead to any other node; solution is impossible' % node)            
            
            if init_graph_is_multigraph and not isMOP and type(mainCr) in [str, np.str_]:
                W = {}
                for out_node in out_nodes:
                    ww = list(Edges[out_node].values())
                    for w in ww:
                        tmp = W.get(out_node, None)
                        if tmp is None:
                            W[out_node] = w
                            continue
                        th = tmp[mainCr]
                        w_main_cr_val = w[mainCr]
                        if isMainCrMin  == (th > w_main_cr_val):
                            W[out_node] = w
                Out_nodes, W = np.array(list(W.keys())), np.array(list(W.values()))
            else:
                W = np.hstack([list(Edges[out_node].values()) for out_node in out_nodes])
                Out_nodes = np.hstack([[out_node] * len(Edges[out_node]) for out_node in out_nodes])
            
            if interalg_gdp:
                rr = np.array([w[MainCr] for w in W])
                if isMainCrMin:
                    rr = -rr
                elif objective[0][2] not in ('max', 'maximum'):
                    self.err('unimplemented for fixed value goal in TSP yet, only min/max is possible for now')

                ind = rr.argsort()
                W = W[ind]
                Out_nodes = Out_nodes[ind]

            lc = 0
            for i, w in enumerate(W):
                if interalg_gdp:
                    edge_ind2x_ind_val[len(EdgesCoords)] = (len(x), lc)
                lc += 1
                EdgesCoords.append((node, Out_nodes[i]))
                EdgesDescriptors.append(w)

                for key, val in w.items():
                    # for undirected:
                    #if node2index[key] < node2index[out_node]: continue
                    Val = val if self.returnToStart or Out_nodes[i] != self.start else 0
                    if key in cr_values:
                        cr_values[key].append(Val)
                    else:
                        cr_values[key] = [Val]
            if interalg_gdp:
                x.append(fd.oovar(domain = np.arange(lc)))            
        
        m = len(EdgesCoords) # new value
        
        if is_glp:
            if type(mainCr) not in (str, np.str_):
                self.err('for the solver "sa" only text name objectives are implemented (e.g. "time", "price")')
            if init_graph_is_multigraph:
                self.err('the solver "sa" cannot handle multigraphs yet')
            if len(Cons) != 0:
                self.err('the solver "sa" cannot handle constrained TSP yet')
            M = np.empty((n, n))
            M.fill(np.nan)
            Cr_values = np.array(cr_values[mainCr])
            isMax = objective[0][-1] in ('max', 'maximum')
            if isMax:
                Cr_values = -Cr_values
            for i, w in enumerate(EdgesDescriptors):
                node_in, node_out = EdgesCoords[i]
                M[node2index[node_in], node2index[node_out]] = Cr_values[i]
            S = np.abs(Cr_values).sum() + 1.0
            
            # TODO: check it
            M[np.isnan(M)] = S
            
            prob = P(lambda x: 0, np.zeros(n), iprint = 1)
            prob.f = lambda x: np.nan if not hasattr(prob, 'ff') else (prob.ff if isMax else -prob.ff)
            prob.M = dict([((i, j), M[i, j]) for i in range(n) for j in range(n) if i != j])
            r = prob.solve(solver, **KW)
            xf = [nodes[j] for j in np.array(r.xf, int)]
            r.nodes = xf#.tolist() 
            if self.start is not None:
                j = r.nodes.index(self.start)
                r.nodes = r.nodes[j:] + r.nodes[:j]
            if self.returnToStart:
                r.nodes += [r.nodes[0]]
            r.edges = [(r.nodes[i], r.nodes[i+1]) for i in range(n-1)] 
            r.Edges = [(r.nodes[i], r.nodes[i+1], graph[r.nodes[i]][r.nodes[i+1]][0]) for i in range(n-1)] 
            if self.returnToStart:
                r.edges.append((r.nodes[-2], r.nodes[0]))
                #print(r.nodes[-1], r.nodes[0], type(r.nodes[-1]), type(r.nodes[0]), graph[2])
                r.Edges.append((r.nodes[-2], r.nodes[0], graph[r.nodes[-2]][r.nodes[0]][0]))
            #r.xf = r.xk = r.nodes
            # TODO: Edges
            
            return r
            
        
        #TODO: fix ooarray lb/ub
        #u = np.array([1] + [fd.oovar(lb=2, ub=n) for i in range(n-1)])
        u = fd.hstack((1, fd.oovars(n-1, lb=2, ub=n)))
        for i in range(1, u.size):
            u[i]('u' + str(i))
            
        
        if is_interalg_raw_mode:
            for i in range(n-1):
                u[1+i].domain = np.arange(2, n+1)
        
        if interalg_gdp: 
            assert len(x) == n
            x = fd.ooarray(x)
            
            # TODO: remove it when proper enum implementation in FD engine will be done
            for i in range(n):
                x[i]._init_domain = x[i].domain
                constraints.append(x[i]-x[i]._init_domain[-1] <= 0)
                x[i].domain = np.arange(int(2 ** np.ceil(np.log2(node_out_edges_num[i]))))
#            for i in range(n-1):
#                u[1+i]._init_domain = u[1+i].domain
#                constraints.append(u[1+i]-u[1+i]._init_domain[-1] <= 0)
#                u[1+i].domain = np.arange(u[1+i]._init_domain[0], u[1+i]._init_domain[0]+int(2 ** np.ceil(np.log2(u[1+i]._init_domain[-1]-u[1+i]._init_domain[0]+1))))
#                u[1+i].ub = u[1+i].domain[-1]

        else:
            x = fd.oovars(m, domain=bool) # new m value
        for i in range(x.size):
            x[i]('x'+str(i))
            
            
#        if init_graph_is_directed:
        dictFrom = dict([(node, []) for node in nodes])
        dictTo = dict([(node, []) for node in nodes])
        for i, edge in enumerate(EdgesCoords):
            From, To = edge
            dictFrom[From].append(i)
            dictTo[To].append(i)

        engine = fd.XOR
        
        # number of outcoming edges = 1
        
        if not interalg_gdp:
            for node, edges_inds in dictFrom.items():
                # !!!!!!!!!! TODO for interalg_raw_mode: and if all edges have sign similar to goal
                if 1 and is_interalg_raw_mode:
                    c = engine([x[j] for j in edges_inds])
                else:
                    nEdges = fd.sum([x[j] for j in edges_inds]) 
                    c =  nEdges >= 1 if self.allowRevisit else nEdges == 1
                constraints.append(c)

        # number of incoming edges = 1
        for node, edges_inds in dictTo.items():
            if len(edges_inds) == 0:
                self.err('input graph has node %s that has no edge from any other node; solution is impossible' % node)
            
            if interalg_gdp:
                x_inds, x_vals = [], []
                for elem in edges_inds:
                    x_ind, x_val = edge_ind2x_ind_val[elem]
                    x_inds.append(x_ind)
                    x_vals.append(x_val)
                c = engine([(x[x_ind] == x_val)(tol = 0.5) for x_ind, x_val in zip(x_inds, x_vals)])
            else:
                if 1 and is_interalg_raw_mode and engine == fd.XOR:
                    c = engine([x[j] for j in edges_inds])
                else:            
                    nEdges = fd.sum([x[j] for j in edges_inds]) 
                    c =  nEdges >= 1 if self.allowRevisit else nEdges == 1
            constraints.append(c)
        
        # MTZ
        for i, (I, J) in enumerate(EdgesCoords):
            ii, jj = node2index[I], node2index[J]
            if ii != 0 and jj != 0:
                if interalg_gdp:
                    x_ind, x_val = edge_ind2x_ind_val[i]
                    c = fd.ifThen((x[x_ind] == x_val)(tol=0.5), u[ii] - u[jj]  <= - 1.0)
                elif is_interalg_raw_mode:
                    c = fd.ifThen(x[i], u[ii] - u[jj]  <= - 1.0)#u[jj] - u[ii]  >= 1)
                else:
                    c = u[ii] - u[jj] + 1 <= (n-1) * (1-x[i])
                constraints.append(c)
        
        # handling objective(s)
        FF = []
        
        for optCrName in usedValues:
            
            tmp = cr_values.get(optCrName, [])

            if len(tmp) == 0:
                self.err('seems like graph edges have no attribute "%s" to perform optimization on it' % optCrName)
            elif len(tmp) != m:
                self.err('for optimization creterion "%s" at least one edge has no this attribute' % optCrName)
            if interalg_gdp:
                F = []
                lc = 0
                for X in x:
                    domain = X._init_domain
                    vals = [tmp[i] for i in range(lc, lc + domain.size)]
                    lc += domain.size
                    #F = sum(x)
                    #F.append(fd.interpolator(domain, vals, k=1, s=0.00000001)(X))
                    F.append(fd.interpolator(domain, vals, k=1)(X))
                    #print(domain, vals)
                F = fd.sum(F)
            else:
                F = fd.sum(x*tmp)
            Funcs[optCrName] = F
        
        for obj in objective:
            FF.append((Funcs[obj[0]](obj[0]) if type(obj[0]) in (str, np.str_) else obj[0](Funcs), obj[1], obj[2]))

        for c in Cons:
            tmp = c(Funcs)
            if type(tmp) in (list, tuple, set):
                constraints += list(tmp)
            else:
                constraints.append(tmp)
        
        startPoint = {x:[0]*(m if not interalg_gdp else n)}
        startPoint.update(dict([(U, i+2) for i, U in enumerate(u[1:])]))

        p = P(FF if isMOP else FF[0][0], startPoint, constraints = constraints)#, fixedVars = fixedVars)
        for param in ('start', 'returnToStart'):
            KW.pop(param, None)
        p.finish = lambda r, p: encode(P, p, r, oo, interalg_gdp, edge_ind2x_ind_val, n, x, EdgesCoords, 
           EdgesDescriptors, m, self)
           
        r = getattr(p, routine)(solver, **KW)
        
        return r

class MOPsolutions(list):
    pass


def encode(P, p, r, oo, interalg_gdp, edge_ind2x_ind_val, n, x, EdgesCoords, 
           EdgesDescriptors, m, self):
    if P != oo.MOP:
        r.ff = p.ff
        if interalg_gdp:
            x_ind_val2edge_ind = dict([(elem[1], elem[0]) for elem in edge_ind2x_ind_val.items()])
            SolutionEdges = [(EdgesCoords[i][0], EdgesCoords[i][1], EdgesDescriptors[i]) for i in [x_ind_val2edge_ind[(ind, x[ind](r))] for ind in range(n)]]
        else:
            SolutionEdges = [(EdgesCoords[i][0], EdgesCoords[i][1], EdgesDescriptors[i]) for i in range(m) if r.xf[x[i]] == 1]
        if len(SolutionEdges) == 0: 
            r.nodes = r.edges = r.Edges = []
            return r
            
        S = dict([(elem[0], elem) for elem in SolutionEdges])
        
        SE = [SolutionEdges[0]]
        for i in range(len(SolutionEdges)-1):
            SE.append(S[SE[-1][1]])
        SolutionEdgesCoords = [(elem[0], elem[1]) for elem in SE]

        nodes = [edge[1] for edge in SolutionEdgesCoords]
        if self.start is not None:
            shift_ind = nodes.index(self.start)
            nodes = nodes[shift_ind:] + nodes[:shift_ind]
        
        if self.returnToStart:
            nodes.append(nodes[0])

        edges = SolutionEdgesCoords[1:] + [SolutionEdgesCoords[0]]
        Edges = SE[1:] + [SE[0]]
        if self.start is not None:
            edges, Edges = edges[shift_ind:] + edges[:shift_ind], Edges[shift_ind:] + Edges[:shift_ind]
        if not self.returnToStart:
            edges, Edges = edges[:-1], Edges[:-1]
        r.nodes, r.edges, r.Edges = nodes, edges, Edges
    else:
        r.solution = 'for MOP see r.solutions instead of r.solution'
        tmp_c, tmp_v = r.solutions.coords, r.solutions.values
#            if interalg_gdp:
#                x_ind_val2edge_ind = dict([(elem[1], elem[0]) for elem in edge_ind2x_ind_val.items()])
#                SolutionEdges = [(EdgesCoords[i][0], EdgesCoords[i][1], EdgesDescriptors[i]) for i in [x_ind_val2edge_ind[(ind, x[ind](r))] for ind in range(n)]]
#            else:
#                SolutionEdges = [(EdgesCoords[i][0], EdgesCoords[i][1], EdgesDescriptors[i]) for i in range(m) if r.xf[x[i]] == 1]
        if interalg_gdp: # default for MOP
            x_ind_val2edge_ind = dict((elem[1], elem[0]) for elem in edge_ind2x_ind_val.items())
            r.solutions = MOPsolutions([[(EdgesCoords[i][0], EdgesCoords[i][1], EdgesDescriptors[i]) for i in [x_ind_val2edge_ind[(ind, x[ind](Point))] for ind in range(n)]] for Point in r.solutions])
        else:# non-default
            r.solutions = MOPsolutions([[(EdgesCoords[i][0], EdgesCoords[i][1], EdgesDescriptors[i]) for i in range(m) if Point[x[i]] == 1] for Point in r.solutions])
            
        r.solutions.values = tmp_v
    r.probType = 'TSP'
class D:
    def __init__(self):
        self.used_vals = set()
    def __getitem__(self, item):
        self.used_vals.add(item)
        return 1.0
    
def getUsedValues(Iterator):
    d = D()
    r = set()
    for elem in Iterator:
        if type(elem) in [str, np.str_]:
            r.add(elem)
        else:
            elem(d)
    r.update(d.used_vals)
    return r
     



"""
usage:
p = someOOclass(..., callback=MyIterFcn, ...)
or
p = ...
p.callback = MyIterFcn
or p.callback = (MyIterFcn1, MyIterFcn2, MyIterFcn3, ..., MyIterFcnN)
or p.callback = [MyIterFcn1, MyIterFcn2, MyIterFcn3, ..., MyIterFcnN]

each user-defined function MyIterFunc should return one of the following:

1. a flag value - 0, 1, True, False
flag = True or 1 means user want to stop calculations
 (r.istop=80, r.msg = 'user-defined' )

2. someRealValue like 15 or 80.15 or 1.5e4 (r.istop=someRealValue, r.msg = 'user-defined')

3. Python list (or tuple) - [istop, msg] (r.istop=istop, r.msg=msg)

works for ralg and lincher, but may doesn't work for some other solvers
(like scipy_cobyla, that has neither native callback nor call gradient)
"""

def MyIterFcn(p):
    # observing non-feasible ralg iter points

    if p.rk > p.contol: # p.rk is current iter max residual
        print '--= non-feasible ralg iter =--'
        print 'itn:',  p.iter
        #however, I inted to change p.iter to p.iter in OpenOpt code soon

        print 'curr f:',  p.fk
        # print 'curr x[:8]:',  p.xk[:8]
        print 'max constraint value',  p.rk

    """
    BTW you can store data in any unique field of p
    for example
    if some_cond:  p.JohnSmith = 15
    else: p.JohnSmith = 0

    However, special field "user" is intended for the purpose:
    p.user.mydata1 = (something)
    # or, for another example:
    if p.iter == 0: p.user.mylist = []
    p.user.mylist.append(something)
    """

    if p.fk < 1.5 and p.rk < p.contol:
        #NB! you could use p.fEnough = 15, p.contol=1e-5 in prob assignment instead
        return (15, 'value obtained is enough' )
        # or
        # return 15 (hence r.istop=15, r.msg='user-defined')
        # or return True (hence r.istop=80, r.msg='user-defined')
        # or return 1 (hence r.istop = 80, r.msg='user-defined')
    else:
        return False
        # or
        # return 0

from openopt import NSP
from numpy import cos,  asfarray,  arange,  sign
N = 75
f = lambda x: sum(1.2 ** arange(len(x)) * abs(x))
df = lambda x: 1.2 ** arange(len(x)) * sign(x)
x0 = cos(1+asfarray(range(N)))

#non-linear constraint c(x) <= 0:
c = lambda x: abs(x[4]-0.8) + abs(x[5]-1.5) - 0.015 

p = NSP(f,  x0,  df=df,  c=c, callback=MyIterFcn,  contol = 1e-5,  maxIter = 1e4,  iprint = 100, xtol = 1e-8, ftol = 1e-8)

#optional:
#p.plot = 1
r = p.solve('ralg')
print r.xf[:8]

"""
-----------------------------------------------------
solver: ralg   problem: unnamed   goal: minimum
 iter    objFunVal    log10(maxResidual)
    0  2.825e+06               0.02
--= non-feasible ralg iter =--
itn: 0
curr f: [ 2824966.83813157]
max constraint value 1.04116752789
--= non-feasible ralg iter =--
itn: 1
curr f: [ 2824973.2896607]
max constraint value 1.75725959686
--= non-feasible ralg iter =--
itn: 2
curr f: [ 2824966.83813157]
max constraint value 1.04116752789
--= non-feasible ralg iter =--
itn: 3
curr f: [ 2824970.22518437]
max constraint value 0.413756712605
--= non-feasible ralg iter =--
itn: 4
curr f: [ 2824969.02632034]
max constraint value 0.0818395397163
--= non-feasible ralg iter =--
itn: 5
curr f: [ 2824969.37414607]
max constraint value 0.0406513995891
--= non-feasible ralg iter =--
itn: 6
curr f: [ 2824969.20023321]
max constraint value 0.00849187556755
--= non-feasible ralg iter =--
itn: 7
curr f: [ 2824969.20119103]
max constraint value 0.00560799704173
--= non-feasible ralg iter =--
itn: 8
curr f: [ 2824969.2065267]
max constraint value 0.00416641026253
--= non-feasible ralg iter =--
itn: 9
curr f: [ 2824969.22185181]
max constraint value 0.0421905566026
--= non-feasible ralg iter =--
itn: 10
curr f: [ 2824969.2065267]
max constraint value 0.00416641026253
--= non-feasible ralg iter =--
itn: 11
curr f: [ 2824969.20952515]
max constraint value 0.00327175155207
  100  2.665e+04            -100.00
  200  4.845e+03            -100.00
  300  1.947e+02            -100.00
  400  9.298e+01            -100.00
  500  5.160e+01            -100.00
  600  2.600e+01            -100.00
  700  1.070e+01            -100.00
  800  6.994e+00            -100.00
  900  5.375e+00            -100.00
 1000  5.375e+00            -100.00
 1094  5.375e+00            -100.00
istop:  4 (|| F[k] - F[k-1] || < ftol)
Solver:   Time Elapsed = 4.62   CPU Time Elapsed = 4.48
objFunValue: 5.3748608 (feasible, max constraint =  0)
[ -1.06086135e-07   5.65437885e-08  -1.29682567e-07   6.12571176e-09
   7.95256506e-01   1.49731951e+00  -1.42518171e-09   4.15961658e-08]
"""



"""
Solving system of equations:
x[0]**3+x[1]**3-9 = 0
x[0]-0.5*x[1] = 0
cos(x[2])+x[0]-1.5 = 0
with some constraints:
150 <= x[2] <= 158
and possible non-linear constraint:
(x[2] - 150.8)**2 <= 1.5

Note:
1. Using Ax <= b constraints is also allowed
2. You can try using equality constraints (h(x)=0, Aeq x = beq) as well.
3. Required function tolerance is p.ftol, constraints tolerance is p.contol,
and hence using h(x)=0 constraints is not 100% same
to some additional f coords
"""

from openopt import SNLE
from numpy import *

# you can define f in several ways:
f = lambda x: (x[0]**3+x[1]**3-9, x[0]-0.5*x[1], cos(x[2])+x[0]-1.5)
#f = (lambda x: x[0]**3+x[1]**3-9, lambda x: x[0]-0.5*x[1], lambda x: cos(x[2])+x[0]-1.5)
# Python list, numpy.array are allowed as well:
#f = lambda x: [x[0]**3+x[1]**3-9, x[0]-0.5*x[1], cos(x[2])+x[0]-1.5]
#or f = lambda x: asfarray((x[0]**3+x[1]**3-9, x[0]-0.5*x[1], cos(x[2])+x[0]-1.5))

#optional: gradient
def df(x):
    df = zeros((3,3))
    df[0,0] = 3*x[0]**2
    df[0,1] = 3*x[1]**2
    df[1,0] = 1
    df[1,1] = -0.5
    df[2,0] = 1
    df[2,2] = -sin(x[2])
    return df

x0 = [8,15, 80]

#w/o gradient:
#p = SNLE(f, x0)

p = SNLE(f, x0, df = df, maxFunEvals = 1e5, iprint = 10, plot=1, ftol = 1e-8, contol=1e-15)

#optional: user-supplied gradient check:
#p.checkdf()

#optional: graphical output, requires matplotlib installed
#p.plot = 1

#set some constraints
p.lb, p.ub = [-inf]*3, [inf]*3
p.lb[2], p.ub[2] = 145, 150

# you could try also comment/uncomment nonlinear constraints:
p.c = lambda x: (x[2] - 146)**2-1.5
# optional: gradient
p.dc = lambda x: asfarray((0, 0, 2*(x[2]-146)))
# also you could set it via p=SNLE(f, x0, ..., c = c, dc = dc)

#optional: user-supplied dc check:
#p.checkdc()

#r = p.solve('nssolve', debug=0, maxIter=1e9)
# using converter to nlp, try to minimize sum(f_i(x)^2):
r = p.solve('nlp:ralg', xlabel='iter', iprint=10, plot=1)

print('solution: %s' % r.xf)
print('max residual: %e' % r.ff)





from openopt import GLP
from numpy import *

# objective function
# (x0 - 1.5)^2 + sin(0.8 * x1^2 + 15)^4 + cos(0.8 * x2^2 + 15)^4 + (x3 - 7.5)^4 -> min
f = lambda x: (x[0]-1.5)**2 + sin(0.8 * x[1] ** 2 + 15)**4 + cos(0.8 * x[2] ** 2 + 15)**4 + (x[3]-7.5)**4

# box-bound constraints lb <= x <= ub
lb, ub = -ones(4),  ones(4)

# linear inequality constraints
# x0 + x3 <= 0.15
# x1 + x3 <= 1.5
# as Ax <= b

A = mat('1 0 0 1; 0 1 0 1') # tuple, list, numpy array etc are OK as well
b = [0.15, 1.5] # tuple, list, numpy array etc are OK as well


# non-linear constraints 
# x0^2 + x2^2 <= 0.15
# 1.5 * x0^2 + x1^2 <= 1.5

c = lambda x: (x[0] ** 2 + x[2] ** 2 - 0.15,  1.5 * x[0] ** 2 + x[1] ** 2 - 1.5)


p = GLP(f, lb=lb, ub=ub, A=A, b=b, c=c, maxIter = 250,  maxFunEvals = 1e5,  maxTime = 30,  maxCPUTime = 30)

r = p.solve('de', mutationRate = 0.15, plot=1)
x_opt,  f_opt = r.xf,  r.ff



import numpy as np
from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.setDefaultIterFuncs import *
from openopt.kernel.ooMisc import LinConst2WholeRepr
#import os
import cplex as CPLEX
from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F, IS_NAN_IN_X

class cplex(baseSolver):
    __name__ = 'cplex'
    __license__ = "free for academic"
    #__authors__ = ''
    #__alg__ = ""
    #__homepage__ = 'http://www.coin-or.org/'
    #__info__ = ""
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'intVars', 'H', 'QC']
    _canHandleScipySparse = True
    __isIterPointAlwaysFeasible__ = False#lambda self, p: p.intVars not in ((), [], None)
    
    preprocessor = lambda *args, **kwargs: None
    #options = ''

    def __init__(self): pass
    def __solver__(self, p):
        for key in (SMALL_DELTA_X, SMALL_DELTA_F,  IS_NAN_IN_X):
            if key in p.kernelIterFuncs:
                p.kernelIterFuncs.pop(key)
        
        # reduce text output
#        try:
#            os.close(1); os.close(2) # may not work for non-Unix OS
#        except:
#            pass
        
        n = p.n
        P = CPLEX.Cplex()
        P.set_results_stream(None)
        
        if np.isfinite(p.maxTime): 
            P.parameters.timelimit.set(p.maxTime)
        
        kwargs = {}
        if hasattr(p, 'intVars') and len(p.intVars)!=0: 
            tmp = np.asarray(['C']*n, dtype=object)
            for v in p.intVars: tmp[v] = 'I'
            kwargs['types'] = ''.join(tmp.tolist())
        
        lb, ub = np.copy(p.lb), np.copy(p.ub)
        lb[lb == -np.inf] = -1e308
        ub[ub == np.inf] = 1e308
        P.variables.add(obj = p.f.tolist(), ub = ub.tolist(), lb = lb.tolist(), **kwargs)
        P.objective.set_sense(P.objective.sense.minimize)
        
        LinConst2WholeRepr(p)
        if p.Awhole is not None:
            senses = ''.join(np.where(p.dwhole == -1, 'L', 'E').tolist())
            P.linear_constraints.add(rhs=np.asarray(p.bwhole).tolist(), senses = senses)
            if p.Awhole.size != 0:
                P.linear_constraints.set_coefficients(zip(*Find(p.Awhole)))
        
        if p.probType.endswith('QP') or p.probType == 'SOCP':
#            assert p.probType in ('QP', 'QCQP','SOCP')
            P.objective.set_quadratic_coefficients(zip(*Find(p.H)))
            for q, u, v in p.QC:
                rows,  cols,  vals = Find(q)
                quad_expr = CPLEX.SparseTriple(ind1=rows, ind2=cols, val = vals)
                lin_expr = CPLEX.SparsePair(ind=np.arange(np.atleast_1d(u).size), val=u)
                P.quadratic_constraints.add(quad_expr = quad_expr, lin_expr = lin_expr, 
                                            rhs = -v if np.isscalar(v) else -np.asscalar(v))

        X = np.nan*np.ones(p.n)
        if p.intVars in ([], (), None):
            class ooContinuousCallback(CPLEX.callbacks.ContinuousCallback):
                def __call__(self):
                    p.iterfcn(X, self.get_objective_value(), self.get_primal_infeasibility())
                    if p.istop != 0: 
                        self.abort()
            P.register_callback(ooContinuousCallback)
        else:
            class ooMIPCallback(CPLEX.callbacks.MIPInfoCallback):
                def __call__(self):
                    #if not self.aborted:
                    p.iterfcn(X, self.get_best_objective_value(), 0.0)
                    if p.istop != 0: 
                        self.abort()

            P.register_callback(ooMIPCallback)
                #get_incumbent_values            
        
        # Temporary walkaround Cplex 12.2.0.0 bug with integers in QP/QCQP
        P.SOS.get_num()
        
        self.preprocessor(P, p)
        p.extras['CplexProb'] = P
        
        P.solve()
        s = P.solution.get_status()
        p.msg = 'Cplex status: "%s"; exit code: %d' % (P.solution.get_status_string(), s)
        try:
            p.xf = p.xk = np.asfarray(P.solution.get_values())
            p.istop = 1000
        except CPLEX.exceptions.CplexError:
            p.xf = p.x0 * np.nan
            p.istop = -1
        
        # TODO: replace by normal OOP solution
        if s == P.solution.status.abort_iteration_limit:
            p.istop = IS_MAX_ITER_REACHED
            p.msg = 'Max Iter has been reached'
        elif s == P.solution.status.abort_obj_limit:
            p.istop = IS_MAX_FUN_EVALS_REACHED
            p.msg = 'max objfunc evals limit has been reached'
        elif s == P.solution.status.abort_time_limit or s == P.solution.status.conflict_abort_time_limit:
            p.istop = IS_MAX_TIME_REACHED
            p.msg = 'max time limit has been reached'
            
#class ooContinuousCallback(CPLEX.callbacks.ContinuousCallback):
#    def __call__(self):
#        print 'current objective:', self.get_objective_value()
#        self.p.iterfcn(f=self.get_objective_value())
#        if self.p.istop != 0: 
#            cplex.terminate()
#            return # is "return" nessesary here?
            
    
def Find(M):
    if isinstance(M, np.ndarray): # numpy array or matrix
        rows, cols = np.where(M)
        vals = M[rows,cols]
    else:
        from scipy import sparse as sp
        assert sp.isspmatrix(M)
        rows, cols, vals = sp.find(M)
    return rows.tolist(), cols.tolist(), vals.tolist()



__docformat__ = "restructuredtext en"

from numpy import diag,  ones, sin, cos, arange, sqrt, vstack, zeros, dot
from openopt import LLSP, NLP

N = 150
C1 = diag(sqrt(arange(N)))
C2 = (1.5+arange(N)).reshape(1, -1) * (0.8+arange(N)).reshape(-1, 1)
C = vstack((C1, C2))
d = arange(2*N)
lb = -2.0+sin(arange(N))
ub = 5+cos(arange(N))

############################LLSP################################
LLSPsolver = 'bvls'
p = LLSP(C, d, lb=lb, ub=ub)
r = p.solve(LLSPsolver)
#############################NLP################################
NLPsolver = 'scipy_lbfgsb'# you could try scipy_tnc or ralg as well
#NLPsolver = 'scipy_tnc'
p2 = LLSP(C, d, lb=lb, ub=ub)
r2=p2.solve('nlp:'+NLPsolver)
##################################################################
print '###########Results:###########'
print 'LLSP solver '+ LLSPsolver + ':', r.ff
print 'NLP solver '+ NLPsolver + ':', r2.ff



from scipy.optimize import anderson2
from numpy import asfarray
from openopt.kernel.baseSolver import baseSolver

class scipy_anderson2(baseSolver):
    __name__ = 'scipy_anderson2'
    __license__ = "BSD"
    #__authors__ = 
    #__alg__ = ""  
    __info__ = """
    solves system of n non-linear equations with n variables. 
    """

    def __init__(self):pass
    def __solver__(self, p):
        
        p.xk = p.x0.copy()
        p.fk = asfarray(max(abs(p.f(p.x0)))).flatten()
        
        p.iterfcn()
        if p.istop:
            p.xf, p.ff = p.xk, p.fk
            return 
        
        try: xf = anderson2(p.f, p.x0, iter = p.maxIter)
        except: 
            p.istop = -1000
            return

        p.xk = p.xf = asfarray(xf)
        p.fk = p.ff = asfarray(max(abs(p.f(xf)))).flatten()
        p.istop = 1000
        p.iterfcn()



"""
You can manage text output in OpenOpt via the following prob parameters:
Prob field name           default value 
iprint                                10
iterObjFunTextFormat         '%0.3e'
finalObjFunTextFormat        '%0.8g'
 
iprint: do text output each iprint-th iteration
You can use iprint = 0 for final output only or iprint < 0 to omit whole output
In future warnings are intended to be shown if iprint >= -1. 
However, some solvers like ALGENCAN have their own text output system, that's hard to suppress, it requires using different approach like, for example, http://permalink.gmane.org/gmane.comp.python.scientific.user/15465

iterObjFunTextFormat: how iter output objFun values are represented
for example, '%0.3e' yields lines like 
 iter    objFunVal
    0  1.947e+03               
   10  1.320e+03            
   ...

finalObjFunTextFormat: how final output objFun value is represented
for example finalObjFunTextFormat='%0.1f' yields
...
objFunValue: 7.9

See Python language documentation for text format specification.
"""



from baseProblem import MatrixProblem
from numpy import ones, inf, dot, zeros, any, all, isfinite, eye
from ooMisc import norm
import NLP

class LLSP(MatrixProblem):
    _optionalData = ['damp', 'X', 'c']
    expectedArgs = ['C', 'd']# for FD it should be Cd and x0
    probType = 'LLSP'
    goal = 'minimum'
    allowedGoals = ['minimum', 'min']
    showGoal = False
    FuncDesignerSign = 'C'
    ftol = 1e-13
    
    def __init__(self, *args, **kwargs):
        MatrixProblem.__init__(self, *args, **kwargs)
        if 'damp' not in kwargs.keys(): self.damp = None
        if 'f' not in kwargs.keys(): self.f = None
        
        if not self._isFDmodel():
            if len(args)>0:
                self.n = args[0].shape[1]
            else:
                self.n = kwargs['C'].shape[1]
            #self.lb = -inf * ones(self.n)
            #self.ub =  inf * ones(self.n)
            if not hasattr(self, 'lb'): self.lb = -inf * ones(self.n)
            if not hasattr(self, 'ub'): self.ub = inf * ones(self.n)
            if self.x0 is None: self.x0 = zeros(self.n)
        else: # is FD model
            if type(self.C) not in (set,  tuple,  list):
                if 'is_oovar' not in dir(self.C): 
                    s = '''
                    Icorrect data type for LLSP constructor, 
                    first argument should be numpy ndarray, 
                    scipy sparse matrix, FuncDesigner oofun or list of oofuns'''
                    self.err(s)
                self.C = [self.C]
            


    def objFunc(self, x):
        r = norm(self.matMultVec(self.C, x) - self.d) ** 2  /  2.0
        if self.damp is not None:
            r += self.damp * norm(x-self.X)**2 / 2.0
        if self.f is not None: r += dot(self.f, x)
        return r

    def llsp2nlp(self, solver, **solver_params):
        if hasattr(self,'x0'): p = NLP.NLP(ff, self.x0, df=dff, d2f=d2ff)
        else: p = NLP.NLP(ff, zeros(self.n), df=dff, d2f=d2ff)
        p.args.f = self # DO NOT USE p.args = self IN PROB ASSIGNMENT!
        self.fill(p)
        self.iprint = -1
        # for LLSP plot is via NLP
        p.show = self.show
        p.plot, self.plot = self.plot, 0
        #p.checkdf()
        r = p.solve(solver, **solver_params)
        self.xf, self.ff, self.rf = r.xf, r.ff, r.rf
        return r

    def _Prepare(self):
        if isinstance(self.d, dict): # FuncDesigner startPoint 
            self.x0 = self.d
        MatrixProblem._Prepare(self)
        if self.isFDmodel:
            self.C, self.d = self._linearOOFunsToMatrices(self.C)
        if not self.damp is None and (not hasattr(self, 'X') or not any(isfinite(self.X))):
            self.X = zeros(self.n)




#def llsp_init(prob, kwargs):
#    if 'damp' not in kwargs.keys(): kwargs['damp'] = None
#    if 'X' not in kwargs.keys(): kwargs['X'] = nan*ones(prob.n)
#    if 'f' not in kwargs.keys(): kwargs['f'] = nan*ones(prob.n)
#
#    if prob.x0 is nan: prob.x0 = zeros(prob.n)


#def ff(x, LLSPprob):
#    r = dot(LLSPprob.C, x) - LLSPprob.d
#    return dot(r, r)
ff = lambda x, LLSPprob: LLSPprob.objFunc(x)
def dff(x, p):
    r = p.matMultVec(p.C.T, p.matMultVec(p.C,x)  - p.d)
    if p.damp is not None and p.damp != 0: r += p.damp*(x - p.X)
    if p.f is not None and all(isfinite(p.f)) : r += p.f
    return r

def d2ff(x, p):
    # TODO: handle sparse!
    r = dot(p.C.T, p.C)
    if not p.damp is None: r += p.damp*eye(x.size)
    return r



__docformat__ = "restructuredtext en"
from numpy import asarray, copy, ravel, isnan, isscalar, asscalar#where
from nonOptMisc import where
from openopt.kernel.Point import Point

class baseSolver:
    def __init__(self):pass
    __name__ = 'Undefined. If you are a user and got the message, inform developers please.'
    __license__ = "Undefined. If you are a user and got the message, inform developers please."
    __authors__ = "Undefined. If you are a user and got the message, inform developers please."
    __alg__ = "Undefined"
    __solver__ = "Undefined. If you are a user and got the message, inform developers please."
    __homepage__ = 'Undefined. Use web search'
    __info__ = 'None'
    _requiresBestPointDetection = False
    _requiresFiniteBoxBounds = False

    """ useStopByException should be turned to False
    for those solvers where passing exceptions through C/Fortran code is buggy
    and cannot be catched by OO
    """
    useStopByException = True

    __optionalDataThatCanBeHandled__ = []
    __isIterPointAlwaysFeasible__ = lambda self, p: p.isUC#TODO: provide possibility of simple True, False
    iterfcnConnected = False
    funcForIterFcnConnection = 'df' # the field is used for non-linear solvers with not-connected iter function
    _canHandleScipySparse = False # True if can handle linear constraints Ax<=b, Aeq x = beq and nonlin cons derivs
    properTextOutput = False
    useLinePoints = False

    # these ones below are used in iterfcn (ooIter.py)
    # to decode input args
    # and can be overdetermined by child class (LP, QP, network etc)
    __expectedArgs__ = ['xk',  'fk',  'rk'] #point, objFunVal, max residual
    def __decodeIterFcnArgs__(self,  p,  *args,  **kwargs):
        """
        decode and assign x, f, maxConstr
        (and/or other fields) to p.iterValues
        """
        fArg  = True

        if len(args)>0 and isinstance(args[0], Point):
            if len(args) != 1: p.err('incorrect iterfcn args, if you see this contact OO developers')
            point = args[0]
            p.xk, p.fk = point.x, point.f()
            p.rk, p.rtk, p.rik = point.mr(True)
            p.nNaNs = point.nNaNs()
            if p.solver._requiresBestPointDetection and (p.iter == 0 or point.betterThan(p._bestPoint)): p._bestPoint = point
        else:
            if len(args)>0: p.xk = args[0]
            elif 'xk' in kwargs.keys(): p.xk = kwargs['xk']
            elif not hasattr(p, 'xk'): p.err('iterfcn must get x value, if you see it inform oo developers')
            if p._baseClassName == 'NonLin': 
                C = p.c(p.xk)
                H = p.h(p.xk)
                p.nNaNs = len(where(isnan(C))[0]) + len(where(isnan(H))[0])
            if p.solver._requiresBestPointDetection:
                currPoint = p.point(p.xk)
                if p.iter == 0 or currPoint.betterThan(p._bestPoint): p._bestPoint = currPoint
            if len(args)>1: p.fk = args[1]
            elif 'fk' in kwargs.keys(): p.fk = kwargs['fk']
            else: fArg = False

            if len(args)>2: 
                #p.pWarn('executing deprecated code, inform developers')
                p.rk = args[2]
            elif 'rk' in kwargs.keys(): 
                #p.pWarn('executing deprecated code, inform developers')
                p.rk = kwargs['rk']
            else:
                p.rk, p.rtk, p.rik = p.getMaxResidual(p.xk, True)
            
        
        p.iterValues.r.append(p.rk)
        if p.probType != 'IP':
            # recalculations are not performed
            p.rk, p.rtk, p.rik = p.getMaxResidual(p.xk, True)
            p.iterValues.rt.append(p.rtk)
            p.iterValues.ri.append(p.rik)
        if p._baseClassName == 'NonLin': p.iterValues.nNaNs.append(p.nNaNs)

        #TODO: handle kwargs correctly! (decodeIterFcnArgs)

#        for key in kwargs.keys():
#            if p.debug: print 'decodeIterFcnArgs>>',  key,  kwargs[key]
#            setattr(p, key, kwargs[key])

        p.iterValues.x.append(copy(p.xk))
        if not p.storeIterPoints and len(p.iterValues.x) > 2:
            p.iterValues.x.pop(0)
        
        if not fArg:
            p.Fk = p.F(p.xk)
            p.fk = copy(p.Fk)
        else:
            if asarray(p.fk).size >1:
                if p.debug and p.iter <= 1: p.warn('please fix solver iter output func, objFuncVal should be single number (use p.F)')
                p.Fk = p.objFuncMultiple2Single(asarray(p.fk))
            else:
                p.Fk = p.fk

        #if p.isObjFunValueASingleNumber: p.Fk = p.fk
        #else: p.Fk = p.objFuncMultiple2Single(fv)

        v = ravel(p.Fk)[0]
        if p.invertObjFunc: v = -v

        p.iterValues.f.append(v)
        
        if not isscalar(p.fk) and p.fk.size == 1:
            p.fk = asscalar(p.fk)
            




from numpy import concatenate, vstack, hstack, inf, ones, zeros, diag, dot, abs, where, max, min
from openopt import NLP, LLSP
from openopt.kernel.ooMisc import norm
from openopt.kernel.setDefaultIterFuncs import IS_MAX_ITER_REACHED

funcDirectionValue = lambda alpha, func, x, direction:  func(x+alpha*direction)

def funcDirectionValueWithMaxConstraintLimit(alpha, func, x, direction, maxConstrLimit, p):
    xx = x+alpha*direction
    mc = p.getMaxResidual(xx)
    if mc > maxConstrLimit:
        return 1e150 * mc
    else:
        return func(xx)
#    cond1 = not(p_LS is None) and hasattr(p_LS, 'xPrevF') and alpha1 == p_LS.xPrevF
#
#    if cond1:
#        constr1 = p.getMaxResidual(x+alpha1*direction)
#        constr2 = p.getMaxResidual(x+alpha2*direction)
#    else:
#        constr2 = p.getMaxResidual(x+alpha2*direction)
#        constr1 = p.getMaxResidual(x+alpha1*direction)
#    if constr2 > constr1 and constr2 > maxConstrLimit: return True
#    elif constr1 > constr2 and constr1 > maxConstrLimit: return False
#
#    if cond1:
#        f1 = p.f(x+alpha1*direction)
#        f2 = p.f(x+alpha2*direction)
#    else:
#        f2 = p.f(x+alpha2*direction)
#        f1 = p.f(x+alpha1*direction)
#    if f1 < f2: return True
#    else: return False

def getConstrDirection(p,  x, regularization = 1e-7):
    c, dc, h, dh, df = p.c(x), p.dc(x), p.h(x), p.dh(x), p.df(x)
    A, Aeq = vstack((dc, p.A)), vstack((dh, p.Aeq))
    b = concatenate((-c, p.b-p.matmult(p.A,x)))
    beq = concatenate((-h, p.beq-p.matmult(p.Aeq,x)))
    lb = p.lb - x
    ub = p.ub - x

    nC = b.size

    #TODO: USE if nC == 0 instead
    if A.size == 0: A_LLS = hstack((zeros((nC, p.n)), diag(ones(nC))))
    else: A_LLS = hstack((A, diag(ones(nC))))

    if Aeq.size == 0:
        Awhole_LLS = vstack((A_LLS, hstack((diag(regularization*ones(p.n)), zeros((p.n, nC))))))
    else:
        Aeq_LLS = hstack((Aeq, zeros((beq.size, nC))))
        Awhole_LLS = vstack((Aeq_LLS, A_LLS, hstack((diag(regularization*ones(p.n)), zeros((p.n, nC))))))

    if p.getMaxResidual(x) > p.contol: dump_x = -p.getMaxConstrGradient(x)
    else: dump_x = -df

    bwhole_LLS = concatenate((beq, b, dump_x))
    lb_LLS = hstack((lb, zeros(nC)))
    ub_LLS = hstack((ub, inf*ones(nC)))
    p_sp = LLSP(Awhole_LLS,  bwhole_LLS,  lb = lb_LLS, ub = ub_LLS, iprint = -1)
    r_sp = p_sp.solve('BVLS', BVLS_inf=1e30)
    return r_sp.xf[:p.n]

class DirectionOptimPoint:
    def __init__(self):
        pass

def getDirectionOptimPoint(p, func, x, direction, forwardMultiplier = 2.0, maxiter = 150, xtol = None, maxConstrLimit = None,  \
                           alpha_lb = 0,  alpha_ub = inf,  \
                           rightLocalization = 0,  leftLocalization = 0, \
                           rightBorderForLocalization = 0, leftBorderForLocalization = None):
    if all(direction==0): p.err('nonzero direction is required')

    if maxConstrLimit is None:
        lsFunc = funcDirectionValue
        args = (func, x, direction)
    else:
        lsFunc = funcDirectionValueWithMaxConstraintLimit
        args = (func, x, direction, maxConstrLimit, p)

    prev_alpha, new_alpha = alpha_lb, min(alpha_lb+0.5, alpha_ub)
    prev_val = lsFunc(prev_alpha, *args)
    for i in range(p.maxLineSearch):
        if lsFunc(new_alpha, *args)>prev_val or new_alpha==alpha_ub: break
        else:
            if i != 0: prev_alpha = min(alpha_lb, new_alpha)
            new_alpha *= forwardMultiplier

    if i == p.maxLineSearch-1: p.debugmsg('getDirectionOptimPoint: maxLineSearch is exeeded')
    lb, ub = prev_alpha, new_alpha

    if xtol is None: xtol = p.xtol / 2.0
    # NB! goldenSection solver ignores x0
    p_LS = NLP(lsFunc, x0=0, lb = lb,  ub = ub, iprint = -1, \
               args=args, xtol = xtol, maxIter = maxiter, contol = p.contol)# contol is used in funcDirectionValueWithMaxConstraintLimit


    r = p_LS.solve('goldenSection', useOOiterfcn=False, rightLocalization=rightLocalization, leftLocalization=leftLocalization, rightBorderForLocalization=rightBorderForLocalization, leftBorderForLocalization=leftBorderForLocalization)
    if r.istop == IS_MAX_ITER_REACHED:
        p.warn('getDirectionOptimPoint: max iter has been exceeded')
    alpha_opt = r.special.rightXOptBorder

    R = DirectionOptimPoint()
    R.leftAlphaOptBorder = r.special.leftXOptBorder
    R.leftXOptBorder = x + R.leftAlphaOptBorder * direction
    R.rightAlphaOptBorder = r.special.rightXOptBorder
    R.rightXOptBorder = x + R.rightAlphaOptBorder * direction
    R.x = x + alpha_opt * direction
    R.alpha = alpha_opt
    R.evalsF = r.evals['f']+i
    return R

from numpy import arccos

def getAltitudeDirection(p, pointTop, point2, point3):
    b = point2 - pointTop
    c = point3 - pointTop
    alpha = dot(c, c-b) / norm(c-b)**2
    h = alpha * b + (1-alpha) * c

    abh = arccos(dot(h, b)/norm(h) /norm(b))
    ach = arccos(dot(h, c)/norm(h) /norm(c))
    abc = arccos(dot(b, c)/norm(b) /norm(c))
    #ahbc = arccos(dot(h,  b-c)/norm(h) /norm(b-c))
    isInsideTriangle = abh+ach-abc<=1e-8
    return h, isInsideTriangle

from openopt.kernel.setDefaultIterFuncs import IS_LINE_SEARCH_FAILED
from openopt.kernel.ooMisc import isSolved

def getBestPointAfterTurn(oldPoint, newPoint, altLinInEq=None, maxLS = 3, maxDeltaX = None, maxDeltaF = None, sf = None, \
                          hs=None, new_bs = True, checkTurnByGradient = False):
    assert altLinInEq is not None
    p = oldPoint.p
    contol = p.contol
    
    altPoint = oldPoint.linePoint(0.5, newPoint)
        

    if maxLS is None:
        maxLS = p.maxLineSearch
        
    assert maxLS != 0
    
    if maxDeltaX is None: 
        if p.iter != 0:
            maxDeltaX = 1.5e4*p.xtol
        else:
            maxDeltaX = 1.5e1*p.xtol
    if maxDeltaF is None: maxDeltaF = 150*p.ftol
    prev_prev_point = newPoint
    
    if checkTurnByGradient:
        direction = newPoint.x-oldPoint.x
        condNotBetter = lambda altPoint, prevPoint: dot(prevPoint._getDirection(p.solver.approach), direction) <= 0.0
        dType = 1
    else:
        condNotBetter = lambda altPoint, prevPoint: not altPoint.betterThan(prevPoint, altLinInEq=altLinInEq)
        dType = 2
            
    for ls in range(maxLS):
        altPoint, prevPoint = oldPoint.linePoint(0.5, altPoint), altPoint
        if sf is not None and sf(altPoint):
            p.debugmsg('Return by sf: ls = %d' % ls)
            return altPoint, prevPoint, -1-ls
            
        #!!! "not betterThan" is used vs "betterThan" because prevPoint can become same to altPoint
        #if not altPoint.betterThan(prevPoint, altLinInEq=altLinInEq):
        if condNotBetter(altPoint, prevPoint):
            if new_bs:
                bestPoint, pointForDilation = prevPoint, prev_prev_point
                #if dType == 1: bestPoint = prev_prev_point
                return bestPoint, pointForDilation, -1-ls
            else:
                return prev_prev_point, -1-ls
        prev_prev_point = prevPoint

        if p.norm(oldPoint.x - altPoint.x) < maxDeltaX: 
            #print 'insufficient step norm during backward search:', p.norm(oldPoint.x - altPoint.x) , maxDeltaX
            break
            
        rr = oldPoint.f() - altPoint.f()
        if (oldPoint.isFeas(True) or altPoint.isFeas(True)) and maxDeltaF/2**15 < abs(rr) < maxDeltaF: break
        
        # TODO: is it worth to implement?
        if (oldPoint.mr_alt() > contol and altPoint.mr_alt() > p.contol) and altPoint.mr_alt() > 0.9*oldPoint.mr(): break 

    # got here if maxLS is exeeded; altPoint is better than prevPoint
    if new_bs:
        bestPoint, pointForDilation = altPoint, prevPoint
        return bestPoint, pointForDilation, -1-ls
    else:
        return prev_prev_point, -1-ls










"""
Example:
Solving nonsmooth problem
|x1| + 1.2|x2| + 1.44|x3| + ... + 1.2^N |xN| -> min
N=75
x0 = [cos(1), cos(2), ..., cos(N)]
x_opt = all-zeros
f_opt = 0
"""

from numpy import *
from openopt import NSP

N = 75
objFun = lambda x: sum(1.2 ** arange(len(x)) * abs(x))

x0 = cos(1+asfarray(range(N)))

p = NSP(objFun, x0, maxFunEvals = 1e7, xtol = 1e-8)
#These assignments are also correct:
#p = NLP(objFun, x0=x0)
#p = NLP(f=objFun, x0=x0)
#p = NLP(ftol = 1e-5, contol = 1e-5, f=objFun, x0=x0)


p.maxIter = 5000

#optional (requires matplotlib installed)
#p.plot = 1
#p.graphics.xlabel = 'cputime'#default: time, case-unsensetive; also here maybe 'cputime', 'niter'

#OPTIONAL: user-supplied gradient/subgradient
p.df = lambda x: 1.2 ** arange(len(x)) * sign(x)

r = p.solve('ralg') # ralg is name of a solver
print('x_opt: %s' % r.xf)
print('f_opt: %f' % r.ff)  # should print small positive number like 0.00056



from baseProblem import MatrixProblem

class STAB(MatrixProblem):
    _optionalData = []
    probType = 'STAB'
    expectedArgs = ['graph']
    allowedGoals = ['graph stability number']
    showGoal = False
    
    def __init__(self, *args, **kw):
        MatrixProblem.__init__(self, *args, **kw) 
        # TODO: simplify it, handle __init_kwargs in parent class
        self.__init_kwargs = kw
        self._immutable = True

    def solve(self, *args, **kw):        
        return set_routine(self,  *args, **kw)

    def manage(self, *args, **kw):
        kw['routine'] = 'manage'
        r = self.solve(*args, **kw)
        return r
    
#    def objFunc(self, x):
#        return dot(self.f, x) + self._c


def set_routine(p,  *args, **kw):
    if len(args) > 1:
        p.err('''
        incorrect number of arguments for solve(), 
        must be at least 1 (solver), other must be keyword arguments''')
    solver = args[0] if len(args) != 0 else kw.get('solver', p.solver)
    routine = kw.pop('routine', 'solve')
    KW = p.__init_kwargs.copy()
    KW.update(kw)
    import FuncDesigner as fd, openopt
    is_interalg = openopt.oosolver(solver).__name__ == 'interalg'
    
    graph = p.graph # must be networkx instance
    nodes = graph.nodes()
    edges = graph.edges()
    
    n = len(nodes)
    
    node2index = dict((node, i) for i, node in enumerate(nodes))
    index2node = dict((i, node) for i, node in enumerate(nodes))
    
    
    x = fd.oovars(n, domain=bool)
    objective = fd.sum(x)
    startPoint = {x:[0]*n}
    
    fixedVars = {}
   
    includedNodes = getattr(kw, 'includedNodes', None)
    if includedNodes is None:
        includedNodes = getattr(p, 'includedNodes', ())
    for node in includedNodes:
        fixedVars[x[node2index[node]]] = 1

    excludedNodes = getattr(kw, 'excludedNodes', None)
    if excludedNodes is None:
        excludedNodes = getattr(p, 'excludedNodes', ())
    for node in excludedNodes:
        fixedVars[x[node2index[node]]] = 0

    if p.probType == 'DSP':
        constraints = []
        engine = fd.OR if is_interalg else lambda List: fd.sum(List) >= 1
        Engine = lambda d, n: engine([x[node2index[k]] for k in (list(d.keys())+[n])]) 
        for node in nodes:
            adjacent_nodes_dict = graph[node]
            if len(adjacent_nodes_dict) == 0:
                fixedVars[x[node2index[node]]] = 1
                continue
            constraints.append(Engine(adjacent_nodes_dict, node))
    else:
        constraints = \
        [fd.NAND(x[node2index[i]], x[node2index[j]]) for i, j in edges] \
        if is_interalg else \
        [x[node2index[i]]+x[node2index[j]] <=1 for i, j in edges]
    
    P = openopt.GLP if is_interalg else openopt.MILP
    goal = 'min' if p.probType == 'DSP' else 'max' 
    p = P(objective, startPoint, constraints = constraints, fixedVars = fixedVars, goal = goal)
    
    for key, val in kw.items():
        setattr(p, key, val)
    
    p.finish = lambda r, p: encode(r, n, x, index2node)
    r = getattr(p, routine)(solver, **KW)
    
    return r

def encode(r, n, x, index2node):
    r.solution = [index2node[i] for i in range(n) if r.xf[x[i]] == 1]
    r.ff = len(r.solution)



from scipy.optimize import fmin_bfgs
from openopt.kernel.ooMisc import isSolved
from openopt.kernel.baseSolver import baseSolver

class scipy_bfgs(baseSolver):
    __name__ = 'scipy_bfgs'
    __license__ = "BSD"
    #__authors__ =
    __alg__ = "BFGS"
    __info__ = 'unconstrained NLP solver'
    iterfcnConnected = True

    def __init__(self):pass
    def __solver__(self, p):

        def iterfcn(x):
            p.xk, p.fk = x, p.f(x)
            p.iterfcn()
            if p.istop: raise isSolved

#        try:
        #p.iterfcn(p.x0)
        p.xk = p.xf = fmin_bfgs(p.f, p.x0, fprime=p.df, disp = 0, gtol=p.gtol, maxiter=p.maxIter, callback=iterfcn)
        p.istop = 1000
#        except isSolved:
#            xf = p.xk
#
#        ff = p.f(xf)
#        p.xk = p.xf = xf
#        p.fk = p.ff = ff
#        p.istop = 1000
#        p.iterfcn()




PythonMin, PythonMax = min, max
import numpy as np

from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F, MAX_NON_SUCCESS, IS_NAN_IN_X
from openopt.kernel.baseSolver import *
from openopt.kernel.MOP import MOPsolutions

from openopt.solvers.UkrOpt.interalgMisc import *
from FuncDesigner import sum as fd_sum, abs as fd_abs, oopoint
from ii_engine import *
from interalgCons import processConstraints
from interalgODE import interalg_ODE_routine
#from interalgT import getTruncatedArrays

from interalgLLR import adjustr4WithDiscreteVariables

bottleneck_is_present = False
try:
    from bottleneck import nanmin
    bottleneck_is_present = True
except ImportError:
    from numpy import nanmin



class interalg(baseSolver):
    __name__ = 'interalg'
    __license__ = "BSD"
    __authors__ = "Dmitrey"
    __alg__ = ""
    __optionalDataThatCanBeHandled__ = [\
    'lb', 'ub', 'c', 'h', 'A', 'Aeq', 'b', 'beq', 'discreteVars', 'QC', 'intVars'\
    ]
    iterfcnConnected = True
    fStart = None
    dataType = np.float64
    #maxMem = '150MB'
    maxNodes = 150000
    minActiveNodes = 15
    maxActiveNodes = 150
    sigma = 0.1 # for MOP, unestablished
    
    _requiresBestPointDetection = True
    
    __isIterPointAlwaysFeasible__ = lambda self, p: \
    p.__isNoMoreThanBoxBounded__() or p.probType in ('MOP', 'IP') #and p.probType != 'IP'
    _requiresFiniteBoxBounds = True

    def __init__(self): 
        self.dataHandling = 'auto'
        
    def __solver__(self, p):
        if not p.isFDmodel:
            p.err('solver %s can handle only FuncDesigner problems' % self.__name__)
            
        isMOP = p.probType == 'MOP'
        if isMOP:
            from interalgMOP import r14MOP
        #isOpt = p.probType in ['NLP', 'NSP', 'GLP', 'MINLP']
        isODE = p.probType == 'ODE'
        isSNLE = p.probType in ('NLSP', 'SNLE')

        if not p.__isFiniteBoxBounded__() and not isODE: 
            p.err('''
            solver %s requires finite lb, ub: 
            lb <= x <= ub 
            (you can use "implicitBounds"),
            e.g. p.implicitBounds = (-100, 100)
            or mere p.implicitBounds = 100
            ''' % self.__name__)
#        if p.fixedVars is not None:
#            p.err('solver %s cannot handle FuncDesigner problems with some variables declared as fixed' % self.__name__)
#        if p.probType in ('LP', 'MILP'):
#            p.err("the solver can't handle problems of type " + p.probType)

        for v in p.freeVarsSet:
            if v.domain is int or v.domain is 'int':
                p.err('interalg cannot handle intVars yet, use domain = [val1, val2, ...,val_k]')

        dataType = self.dataType
        if type(dataType) == str:
            if not hasattr(np, dataType):
                p.pWarn('your architecture has no type "%s", float64 will be used instead' % dataType)
                dataType = 'float64'
            dataType = getattr(np, dataType)
            self.dataType = dataType
        
        vv = list(p._freeVarsList)
        p._freeVarsDict = dict((v, i) for i, v in enumerate(vv))
        x0 = dict((v, p._x0[v]) for v in vv)
        domain = oopoint([(v, [atleast_1d(p.lb[i]), atleast_1d(p.ub[i])]) for i,  v in enumerate(vv)], skipArrayCast = True)
        domain.dictOfFixedFuncs = p.dictOfFixedFuncs
        domain._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
        
        isIP = p.probType == 'IP'
        if isIP:
            pb = r14IP
            p._F = asarray(0, self.dataType)
            p._residual = 0.0
            f_int = p.user.f[0].interval(domain, self.dataType)
            p._r0 = prod(p.ub-p.lb) * (f_int.ub - f_int.lb)
            p._volume = 0.0
            p.kernelIterFuncs.pop(IS_NAN_IN_X)
        elif isMOP:
            pb = r14MOP
        else:
            pb = r14

        
        for val in p._x0.values():
            if isinstance(val,  (list, tuple, np.ndarray)) and len(val) > 1:
                p.pWarn('''
                solver %s currently can handle only single-element variables, 
                use oovars(n) instead of oovar(size=n),
                elseware correct result is not guaranteed
                '''% self.__name__)

        
        for val in x0.values():
            if isinstance(val,  (list, tuple, np.ndarray)) and len(val) > 1:
                p.err('''
                solver %s currently can handle only single-element variables, 
                use oovars(n) instead of oovar(size=n)'''% self.__name__)

        point = p.point
        
        p.kernelIterFuncs.pop(SMALL_DELTA_X, None)
        p.kernelIterFuncs.pop(SMALL_DELTA_F, None)
        p.kernelIterFuncs.pop(MAX_NON_SUCCESS, None)
        
        if not bottleneck_is_present and not isODE:
                p.pWarn('''
                installation of Python module "bottleneck" 
                (http://berkeleyanalytics.com/bottleneck,
                available via easy_install, takes several minutes for compilation)
                could speedup the solver %s''' % self.__name__)
        
        n = p.n
        
        maxSolutions = p.maxSolutions
        if maxSolutions == 0: maxSolutions = 10**50
        if maxSolutions != 1 and p.fEnough != -np.inf:
            p.warn('''
            using the solver interalg with non-single solutions mode 
            is not ajusted with fEnough stop criterium yet, it will be omitted
            ''')
            p.kernelIterFuncs.pop(FVAL_IS_ENOUGH)
        
        nNodes = []        
        p.extras['nNodes'] = nNodes
        nActiveNodes = []
        p.extras['nActiveNodes'] = nActiveNodes

        Solutions = Solution()
        Solutions.maxNum = maxSolutions
        Solutions.solutions = []
        Solutions.coords = np.array([]).reshape(0, n)
        p.solutions = Solutions
        
        lb, ub = asarray(p.lb, dataType).copy(), asarray(p.ub, dataType).copy()

        fTol = p.fTol
        if isIP or isODE:
            if p.ftol is None:
                if fTol is not None:
                    p.ftol = fTol
                else:
                    p.err('interalg requires user-supplied ftol (required precision)')
            if fTol is None: fTol = p.ftol
            elif fTol != p.ftol:
                p.err('you have provided both ftol and fTol')

        if fTol is None and not isMOP: # TODO: require tols for MOP
            fTol = 1e-7
            p.warn('solver %s requires p.fTol value (required objective function tolerance); 10^-7 will be used' % self.__name__)

        xRecord = 0.5 * (lb + ub)
        adjustr4WithDiscreteVariables(xRecord.reshape(1, -1), p)

        r40 = np.inf
        
        y = lb.reshape(1, -1)
        e = ub.reshape(1, -1)
        r41 = np.inf

        # TODO: maybe rework it, especially for constrained case
        fStart = self.fStart
        
        # TODO: remove it after proper SNLE handling implementation
        if isSNLE:
            r41 = 0.0
#            asdf1 = None
            eqs = [fd_abs(elem) for elem in p.user.f]
            asdf1 = fd_sum(eqs)
            
            # temporary change, TODO: rework it
            asdf1.resolveSchedule = asdf1._getDep()
            
            # TODO: check it, for reducing calculations
            #C.update([elem == 0 for elem in p.user.f])
        elif isMOP:
            asdf1 = p.user.f
            Solutions.F = []
            if point(p.x0).isFeas(altLinInEq=False):
                Solutions.solutions.append(p.x0.copy())
                Solutions.coords = asarray(Solutions.solutions)
                Solutions.F.append(p.f(p.x0))
                p._solutions = Solutions
        elif not isODE:
            asdf1 = p.user.f[0]
            
            #if p.fOpt is not None:  fOpt = p.fOpt
            if p.goal in ('max', 'maximum'):
                asdf1 = -asdf1
                if p.fOpt is not None:
                    p.fOpt = -p.fOpt
            
            if fStart is not None and fStart < r40: 
                r41 = fStart
                
            for X0 in [point(xRecord), point(p.x0)]:
                if X0.isFeas(altLinInEq=False) and X0.f() < r40:
                    r40 = X0.f()

            if p.isFeas(p.x0):
                tmp = asdf1(p._x0)
                if  tmp < r41:
                    r41 = tmp
                
            if p.fOpt is not None:
                if p.fOpt > r41:
                    p.warn('user-provided fOpt seems to be incorrect, ')
                r41 = p.fOpt

#        if isSNLE:
#            if self.dataHandling == 'raw':
#                p.pWarn('''
#                    this interalg data handling approach ("%s") 
#                    is unimplemented for SNLE yet, dropping to "sorted"'''%self.dataHandling)
#            
#            # handles 'auto' as well
#            self.dataHandling ='sorted'

        #from FuncDesigner.ooFun import BooleanOOFun, SmoothFDConstraint
        
        if p.hasLogicalConstraints:
            if self.dataHandling == 'sorted': 
                p.warn("interalg: for general logical constraints only dataHandling='raw' mode works")
            self.dataHandling = 'raw'
        
        if not isMOP and not p.hasLogicalConstraints:
            p._isOnlyBoxBounded = p.__isNoMoreThanBoxBounded__() 

        if self.dataHandling == 'auto':
            if isIP or isODE:
                self.dataHandling = 'sorted'
            elif isMOP or p.hasLogicalConstraints:
                self.dataHandling = 'raw'
            else:
                r = p.user.f[0].interval(domain, self.dataType)
                M = np.max((np.max(np.atleast_1d(np.abs(r.lb))), np.max(np.atleast_1d(np.abs(r.ub)))))
                for (c, func, lb, ub, tol) in p._FD.nonBoxCons:#[Elem[1] for Elem in p._FD.nonBoxCons]:

                    # !!!!!!!!!!!!!!!!!!!! check it - mb 2nd condition is incorrect
                    #if isinstance(c, BooleanOOFun) and not isinstance(c, SmoothFDConstraint): continue
                    if hasattr(c,'_unnamedBooleanOOFunNumber'):
                        continue
                    
                    r = func.interval(domain, self.dataType)
                    M = np.max((M, np.max(np.atleast_1d(np.abs(r.lb)))))
                    M = np.max((M, np.max(np.atleast_1d(np.abs(r.ub)))))
                
                self.dataHandling = 'raw' if M < 1e5 else 'sorted'
                
                # TODO: is it required yet?
                if asdf1.isUncycled and p._isOnlyBoxBounded and np.all(np.isfinite(p.user.f[0].interval(domain).lb)):
                    #maxNodes = 1
                    self.dataHandling = 'sorted'
                if p.iprint >= 0:
                    p.disp('interalg parameter dataHandling has been autoselected to "%s"' % self.dataHandling)

            #self.dataHandling = 'sorted' if isIP or (p.__isNoMoreThanBoxBounded__() and n < 50) else 'raw'

        self.maxActiveNodes = int(self.maxActiveNodes)
#        if self.maxActiveNodes < 2:
#            p.warn('maxActiveNodes should be at least 2 while you have provided %d. Setting it to 2.' % self.maxActiveNodes)
        self.maxNodes = int(self.maxNodes)
        
        g = np.inf
        C = p._FD.nonBoxConsWithTolShift
        C0 = p._FD.nonBoxCons
        
        if isSNLE:
            C += [(elem==0, elem, -(elem.tol if elem.tol != 0 else p.ftol), (elem.tol if elem.tol != 0 else p.ftol)) for elem in p.user.f]
            C0 += [(elem==0, elem, 0, 0, (elem.tol if elem.tol != 0 else p.ftol)) for elem in p.user.f]
        
        # TODO: hanlde fixed variables here
        varTols = p.variableTolerances
        if Solutions.maxNum != 1:
            if not isSNLE:
                p.err('''
                "search several solutions" mode is unimplemented
                for the prob type %s yet''' % p.probType)
            if any(varTols == 0):
                p.err('''
                for the mode "search all solutions" 
                you have to provide all non-zero tolerances 
                for each variable (oovar)
                ''')
            
        pnc = 0
        an = []
        maxNodes = self.maxNodes
        
        # TODO: change for constrained probs
        _s = atleast_1d(inf)
        
        if isODE or (isIP and p.n == 1):
            interalg_ODE_routine(p, self)
            return
        
        #_in = np.array([], object)
        _in = []
        while 1:
#            p.iprint=1
            if len(C0) != 0: # SNLE also can have constraints
                y, e, nlhc, residual, definiteRange, indT, _s = processConstraints(C0, y, e, _s, p, dataType)
            else:
                nlhc, residual, definiteRange, indT = None, None, True, None
#            P = array([-0.63521194458007812, -0.3106536865234375, 0.0905609130859375, 0.001522064208984375, -0.69999999999999996, -0.99993896484375, 0.90000152587890625, 1.0, 4.0])
#            print('-', p.iter, hasPoint(y, e, P), pointInd(y, e, P))
            if y.size != 0:
                an, g, fo, _s, Solutions, xRecord, r41, r40 = \
                pb(p, nlhc, residual, definiteRange, y, e, vv, asdf1, C, r40, g, \
                             nNodes, r41, fTol, Solutions, varTols, _in, \
                             dataType, maxNodes, _s, indT, xRecord)
                if _s is None:
                    break
            else:
                an = _in
                fo = 0.0 if isSNLE or isMOP else \
                PythonMin(r41, \
                          r40 - (fTol if Solutions.maxNum == 1 else 0.0))
            pnc = PythonMax(pnc, 
                                    len(np.atleast_1d(an) if type(an) == np.ndarray else an))
            
            y, e, _in, _s = \
                func12(an, self.maxActiveNodes, p, Solutions, vv, varTols, np.inf if isIP else fo)

            nActiveNodes.append(y.shape[0]/2)
            if y.size == 0: 
                if len(Solutions.coords) > 1:
                    p.istop, p.msg = 1001, 'all solutions have been obtained'
                else:
                    p.istop, p.msg = 1000, 'solution has been obtained'
                break            
            ############# End of main cycle ###############
            
        if not isSNLE and not isIP and not isMOP:
            p_xk = p.point(p.xk)
            p.iterfcn(p._bestPoint if p._bestPoint.betterThan(p_xk) else p_xk)

        
        ff = p.fk # ff may be not assigned yet
#        ff = p._bestPoint.f()
#        p.xk = p._bestPoint.x
        if isIP: 
            p.xk = array([np.nan]*p.n)
            p.rk = p._residual
            p.fk = p._F
        
        isFeas = len(Solutions.F) != 0 if isMOP else p.isFeas(p.xk) if not isIP else p.rk < fTol
        
        if not isFeas and p.istop > 0:
            p.istop, p.msg = -1000, 'no feasible solution has been obtained'
        
        if isSNLE:
            lf = inf
        else:
            lf = array([t.key for t in an])#.flatten()
            if lf.size != 0:
                g = nanmin([nanmin(lf), g])

        if not isMOP:

            cond_rTol = \
            ff - g <= p.rTol * PythonMin(abs(ff), abs(g))
            
            cond_fTol = ff - g < fTol
            
            p.extras['isRequiredPrecisionReached'] = \
            True if(cond_fTol or cond_rTol) and isFeas else False
            # and (k is False or (isSNLE and (p._nObtainedSolutions >= maxSolutions or maxSolutions==1))) 

        if not isMOP and not p.extras['isRequiredPrecisionReached'] and p.istop > 0:
            p.istop = -1
            p.msg = 'required precision is not guarantied' if p.probType not in ('NLP', 'GLP', 'NSP') else\
            'required precision (fTol = %g, rTol = %g) is not guarantied' % (fTol, p.rTol)
            
        # TODO: simplify it
        if not isMOP:
            tmp = [nanmin(np.hstack((ff, g, lf))), np.asscalar(np.array(ff))]
            if p.goal in ['max', 'maximum']: tmp = (-tmp[1], -tmp[0])
            p.extras['extremumBounds'] = tmp if not isIP else 'unimplemented for IP yet'
        
        if isMOP or p.maxSolutions != 1:
            p.solutions = (MOPsolutions if isMOP else list)([p._vector2point(s) for s in Solutions.coords])
        if isMOP:
            for i, s in enumerate(p.solutions):
                s.useAsMutable = True
                for v, val in s.items():
                    if v.fields != ():
                        s[v] = dict((field, v.aux_domain[int(val)][j]) for j, field in enumerate(v.fields))
                for j, goal in enumerate(p.user.f):
                    s[goal] = Solutions.F[i][j]
                s.useAsMutable = False
            p.solutions.values = np.asarray(Solutions.F)
            p.solutions.coords = Solutions.coords
        if isSNLE and p.maxSolutions != 1:
            for v in p._stringVars:
                for elem in r.solutions:
                    elem.useAsMutable = True
                    elem[v] = v.aux_domain[elem[v]]
                    elem.useAsMutable = False
        if p.iprint >= 0 and not isMOP:
            s = 'Solution with required tolerance %0.1e \n is%s guarantied' \
            %(fTol, '' if p.extras['isRequiredPrecisionReached'] else ' NOT') \
            if p.probType not in ('NLP', 'GLP', 'NSP', 'MINLP')\
            else 'Solution with required tolerances fTol = %g, rTol = %g \n is%s guarantied' \
            %(fTol, p.rTol, '' if p.extras['isRequiredPrecisionReached'] else ' NOT')
            
            if not isIP and p.maxSolutions == 1:
                s += ' (obtained precision: %0.1e)' % np.abs(tmp[1]-tmp[0])
            if not p.extras['isRequiredPrecisionReached'] and pnc == self.maxNodes: s += '\nincrease maxNodes (current value %d)' % self.maxNodes
            p.info(s)

class Solution:
    pass
    




from baseProblem import NonLinProblem
from numpy import inf

from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F
class MOP(NonLinProblem):
    _optionalData = ['lb', 'ub', 'A', 'Aeq', 'b', 'beq', 'c', 'h']
    showGoal = True
    goal = 'weak Pareto front'
    probType = 'MOP'
    allowedGoals = ['weak Pareto front', 'strong Pareto front', 'wpf', 'spf']
    isObjFunValueASingleNumber = False
    expectedArgs = ['f', 'x0']
    _frontLength = 0
    _nIncome = 0
    _nOutcome = 0
    
    iprint = 1
    
    def __init__(self, *args, **kwargs):
        NonLinProblem.__init__(self, *args, **kwargs)
        self.nSolutions = 'all'
        self.kernelIterFuncs.pop(SMALL_DELTA_X, None)
        self.kernelIterFuncs.pop(SMALL_DELTA_F, None)
        self.data4TextOutput = ['front length', 'income', 'outcome', 'log10(maxResidual)']
        f = self.f
        i = 0
        targets = []
        while True:
            if len(f[i:]) == 0: break
            func = f[i]
            if type(func) in (list, tuple):
                F, tol, val = func
                i += 1
            else:
                F, tol, val = f[i], f[i+1], f[i+2]
                i += 3
            t = target()
            t.func, t.tol = F, tol
            t.val = val if type(val) != str \
            else inf if val in ('max', 'maximum') \
            else -inf if val in ('min', 'minimum') \
            else self.err('incorrect MOP func target')
            targets.append(t)
        self.targets = targets
        self.f = [t.func for t in targets]
        self.user.f = self.f

    def objFuncMultiple2Single(self, fv):
        return 0#(fv ** 2).sum()

    def solve(self, *args, **kw):
#        if self.plot or kw.get('plot', False):
#            self.warn('\ninteractive graphic output for MOP is unimplemented yet and will be turned off')
#            kw['plot'] = False
        self.graphics.drawFuncs = [mop_iterdraw]
        r = NonLinProblem.solve(self, *args, **kw)
        r.plot = lambda *args, **kw: self._plot(**kw)
        r.__call__ = lambda *args, **kw: self.err('evaluation of MOP result on arguments is unimplemented yet, use r.solutions')
        r.export = lambda *args, **Kw: _export_to_xls(self, r, *args, **kw)
        T0 = self.targets[0]
        isTSP = r.probType == 'TSP'
        
        #TODO: mb simplify the code below
#        c = lambda elem, f: elem.get(f)
        if T0.val == -inf:
            #T0.func.name was added for KSP and some similar probs
            keyfunc = lambda elem: \
            elem.get(T0.func,  elem.get(T0.func.name, None)) if not isTSP else\
            sum(edge[2][T0.func.name] for edge in elem)
        elif T0.val == inf:
            keyfunc = lambda elem: \
            -elem.get(T0.func, elem.get(T0.func.name, None)) if not isTSP else\
            -sum(edge[2][T0.func.name] for edge in elem)
        else:
            keyfunc = lambda elem: \
            abs(T0.val - elem.get(T0.func, elem.get(T0.func.name, None))) if not isTSP else\
            abs(T0.val - sum(edge[2][T0.func.name] for edge in elem))

        r.solutions.sort(key=keyfunc)
        for v in self._stringVars:
            for elem in r.solutions:
                elem.useAsMutable = True
                elem[v] = v.reverse_aux_domain[int(elem[v])]
                elem.useAsMutable = False
        return r

    def _plot(self, **kw):
        from numpy import asarray, atleast_1d, array_equal
        S = self.solutions
        if type(S)==list and len(S) == 0: return
        tmp = asarray(self.solutions.F if 'F' in dir(self.solutions) else self.solutions.values)
        from copy import deepcopy
        kw2 = deepcopy(kw)
        useShow = kw2.pop('show', True)
        if not useShow and hasattr(self, '_prev_mop_solutions') and array_equal(self._prev_mop_solutions, tmp):
            return
        self._prev_mop_solutions = tmp.copy()
        if tmp.size == 0:
            if self.isFinished:
                self.disp('no solutions, nothing to plot')
            return
        try:
            import pylab
        except:
            self.err('you should have matplotlib installed')
        pylab.ion()
        if self.nf != 2:
            self.err('MOP plotting is implemented for problems with only 2 goals, while you have %d' % self.nf)
        X, Y = atleast_1d(tmp[:, 0]), atleast_1d(tmp[:, 1])

        useGrid = kw2.pop('grid', 'on')
        
        if 'marker' not in kw2: 
            kw2['marker'] = (5, 1, 0)
        if 's' not in kw2:
            kw2['s']=[150]
        if 'edgecolor' not in kw2:
            kw2['edgecolor'] = 'b'
        if 'facecolor' not in kw2:
            kw2['facecolor'] = '#FFFF00'#'y'
            
        pylab.scatter(X, Y, **kw2)
        
        pylab.grid(useGrid)
        t0_goal = 'min' if self.targets[0].val == -inf else 'max' if self.targets[0].val == inf else str(self.targets[0].val)
        t1_goal = 'min' if self.targets[1].val == -inf else 'max' if self.targets[1].val == inf else str(self.targets[1].val)
        
        pylab.xlabel(self.user.f[0].name + ' (goal: %s    tolerance: %s)' %(t0_goal, self.targets[0].tol))
        pylab.ylabel(self.user.f[1].name + ' (goal: %s    tolerance: %s)' %(t1_goal, self.targets[1].tol))
        
        pylab.title('problem: %s    goal: %s' %(self.name, self.goal))
        figure = pylab.gcf()
        from openopt import __version__ as ooversion
        figure.canvas.set_window_title('OpenOpt ' + ooversion)
        
        pylab.hold(0)
        pylab.draw()
        if useShow: 
            pylab.ioff()
            pylab.show()

def mop_iterdraw(p):
    p._plot(show=False)

TkinterIsInstalled = True
import platform
if platform.python_version()[0] == '2': 
    # Python2
    try:
        from Tkinter import Tk
        from tkFileDialog import asksaveasfilename
    except:
        TkinterIsInstalled = False
else: 
    # Python3
    try:
        from tkinter import Tk
        from tkinter.filedialog import asksaveasfilename
    except:
        TkinterIsInstalled = False

def _export_to_xls(p, r, *args, **kw):
    try:
        import xlwt
    except:
        s = '''
        To export OpenOpt MOP result into xls file
        you should have Python module "xlwt" installed,
        (http://www.python-excel.org),
        available via easy_install xlwt 
        or Linux apt-get python-xlwt
        '''
        p.err(s)
    if len(args) != 0:
        xls_file = args[0]
    elif TkinterIsInstalled:
        root = Tk()
        root.withdraw()
        import os
        hd = os.getenv("HOME")
        xls_file = asksaveasfilename(defaultextension='.xls', initialdir = hd, filetypes = [('xls files', '.xls')])
        root.destroy()
        if xls_file in (None, ''): 
            return
    else:
        p.err('''
        you should either provide xls file name for data output 
        or have tkinter installed to set it via GUI window''')
        
#    xls_file = asksaveasfilename(defaultextension='.xls', initialdir = self.hd, filetypes = [('xls files', '.xls')])
#    if xls_file in (None, ''):
#        return

    nf = p.nf
    target_funcs = [t.func for t in p.targets]
    vars4export = set(p._freeVarsList).difference(target_funcs)
    vars4export = list(vars4export)
    vars4export.sort(key = lambda v: v._id)
    nv = len(vars4export)
    R = [[] for i in range(nv + nf)]
    Names = [t.name for t in target_funcs] + [v.name for v in vars4export]
    Keys = target_funcs + vars4export
    for elem in r.solutions:
        for i, key in enumerate(Keys):
            R[i].append(elem[key])
    from numpy import asarray
    R = asarray(R)
    
    L = len(r.solutions)
    
    wb = xlwt.Workbook()
    

    ws = wb.add_sheet('OpenOpt MOP result')
    from openopt import __version__ as ver
    i = 0
    ws.write(i, 0, 'OpenOpt ver')
    ws.write(i, 1, ver)
    i += 1
    ws.write(i, 0, 'Solver')
    ws.write(i, 1, p.solver.__name__)
    i += 1
    ws.write(i, 0, 'Prob name')
    ws.write(i, 1, p.name)
    i += 1
    ws.write(i, 0, 'Prob type')
    ws.write(i, 1, p.probType)
    i += 1
    ws.write(i, 0, 'Time, s')
    ws.write(i, 1, str(int(r.elapsed['solver_time'])))
    i += 1
    ws.write(i, 0, 'CPU Time, s')
    ws.write(i, 1, str(int(r.elapsed['solver_cputime'])))
    i += 1
    ws.write(i, 0, 'N solutions')
    ws.write(i, 1, str(L))
       
    style1 = xlwt.easyxf("""
         font:
             name Times New Roman,
             colour_index black;
         pattern:
             back_colour yellow,
             pattern thick_forward_diag,
             fore-colour yellow
         """) 
    for i in range(nf):
        ws.write(0, 3+i, Names[i], style1)
        for j in range(L):
            ws.write(1+j, 3+i, R[i, j], style1)

    for i in range(nf, nf + nv):
        ws.write(0, 3+i, Names[i])
        for j in range(L):
            ws.write(1+j, 3+i, R[i, j])


    wb.save(xls_file)
    p.disp('export MOP %s result to xls file finished' % p.name)
    
class target:
    pass
    
class MOPsolutions(list):
    pass



"""
Example of solving Mini-Max Problem
via converter to NLP

latter works via solving NLP
t -> min
subjected to
t >= f0(x)
t >= f1(x)
...
t >= fk(x)

Splitting f into separate funcs could benefit some solvers
(ralg, algencan; see NLP docpage for more details)
but is not implemented yet
"""
from numpy import *
from openopt import *

n = 15

f1 = lambda x: (x[0]-15)**2 + (x[1]-80)**2 + (x[2:]**2).sum()
f2 = lambda x: (x[1]-15)**2 + (x[2]-8)**2 + (abs(x[3:]-100)**1.5).sum()
f3 = lambda x: (x[2]-8)**2 + (x[0]-80)**2 + (abs(x[4:]+150)**1.2).sum()
f = [f1, f2, f3]

# you can define matrices as numpy array, matrix, Python lists or tuples

#box-bound constraints lb <= x <= ub
lb = [0]*n
ub = [15,  inf,  80] + (n-3) * [inf]

# linear ineq constraints A*x <= b
A = array([[4,  5,  6] + [0]*(n-3), [80,  8,  15] + [0]*(n-3)])
b = [100,  350]

# non-linear eq constraints Aeq*x = beq
Aeq = [15,  8,  80] + [0]*(n-3)
beq = 90

# non-lin ineq constraints c(x) <= 0
c1 = lambda x: x[0] + (x[1]/8) ** 2 - 15
c2 = lambda x: x[0] + (x[2]/80) ** 2 - 15
c = [c1, c2]
#or: c = lambda x: (x[0] + (x[1]/8) ** 2 - 15, x[0] + (x[2]/80) ** 2 - 15)

# non-lin eq constraints h(x) = 0
h = lambda x: x[0]+x[2]**2 - x[1]

x0 = [0, 1, 2] + [1.5]*(n-3)
p = MMP(f,  x0,  lb = lb,  ub = ub,   A=A,  b=b,   Aeq = Aeq,  beq = beq,  c=c,  h=h, xtol = 1e-6, ftol=1e-6)
# optional, matplotlib is required:
p.plot=1
r = p.solve('nlp:ipopt', iprint=50, maxIter=1e3)
print 'MMP result:',  r.ff

### let's check result via comparison with NSP solution
F= lambda x: max([f1(x),  f2(x),  f3(x)])
p = NSP(F,  x0, iprint=50, lb = lb, ub = ub,  c=c,  h=h,  A=A,  b=b,  Aeq = Aeq,  beq = beq, xtol = 1e-6, ftol=1e-6)
r_nsp = p.solve('ipopt', maxIter = 1e3)
print 'NSP result:',  r_nsp.ff,  'difference:', r_nsp.ff - r.ff
"""
-----------------------------------------------------
solver: ipopt   problem: unnamed   goal: minimax
 iter    objFunVal    log10(maxResidual)
    0  1.196e+04               1.89
   50  1.054e+04              -8.00
  100  1.054e+04              -8.00
  150  1.054e+04              -8.00
  161  1.054e+04              -6.10
istop:  1000
Solver:   Time Elapsed = 0.93   CPU Time Elapsed = 0.88
objFunValue: 10536.481 (feasible, max constraint =  7.99998e-07)
MMP result: 10536.4808622
-----------------------------------------------------
solver: ipopt   problem: unnamed   goal: minimum
 iter    objFunVal    log10(maxResidual)
    0  1.196e+04               1.89
   50  1.054e+04              -4.82
  100  1.054e+04             -10.25
  150  1.054e+04             -15.35
StdOut: Problem solved
[PyIPOPT] Ipopt will use Hessian approximation.
[PyIPOPT] nele_hess is 0
  192  1.054e+04             -13.85
istop:  1000
Solver:   Time Elapsed = 2.42   CPU Time Elapsed = 2.42
objFunValue: 10536.666 (feasible, max constraint =  1.42109e-14)
NSP result: 10536.6656339 difference: 0.184771728482
"""



PythonAny = any
PythonAll = all
from numpy import isscalar, ndarray, asscalar
from oologfcn import OpenOptException

    
def formDictOfFixedFuncs(oof, p):
    dictOfFixedFuncs, startPoint = p.dictOfFixedFuncs, p._x0
    from FuncDesigner.distribution import Stochastic
    oof = p._dictOfRedirectedFuncs.get(oof, oof)
    dep = oof.Dep#set([oof]) if oof.is_oovar else oof._getDep()
    # TODO: rework it as mixin of fixed and stoch variables
    #if PythonAll((isinstance(startPoint[t], stochasticDistribution) or areFixed(set([t]))) for t in dep):
#    cond_st = not PythonAny(isinstance(startPoint[t], stochasticDistribution) for t in dep) \
#    or 
    isFixedNonStoch = p._isFixedNonStoch
    x0 = p._x0
#    stochEngines = ('mean', 'std','var')

    cond = lambda t: isFixedNonStoch(t) or (isinstance(x0[t], Stochastic) and oof.engine == 'mean')
    c1 = PythonAll(cond(t) for t in dep)
    if c1:
        dictOfFixedFuncs[oof] = oof(startPoint)#TODO: add fixity ID as additional argument to the func

def linear_render(f, D, Z):
    #TODO: linear render for linear parts of sum of oofuns
    import FuncDesigner as fd
    if len(D) == 0:
        raise OpenOptException('probably you try to optimize a fixed constant')
    if f.is_oovar: 
        return f
    ff = f(Z)
    name, tol, _id = f.name, f.tol, f._id
    tmp = [(v if isscalar(val) and val == 1.0 else v * (val if type(val) != ndarray or val.ndim < 2 else val.flatten())) \
    for v, val in D.items()]

    c = ff if isscalar(ff) or ff.ndim <= 1 else asscalar(ff)
    if c != 0: tmp.append(c)
    f = tmp[0] if len(tmp) == 1 else tmp[0]+tmp[1] if len(tmp) == 2 else fd.sum(tmp)
    
    f.name, f.tol, f._id = name, tol, _id
    
    return f



def hasStochDep(s, x0):
    from FuncDesigner import oofun, ooarray
    from FuncDesigner.distribution import stochasticDistribution
#    print [isinstance(t, stochasticDistribution) for t in s.Dep]
    cond1 = (isinstance(s, oofun) and PythonAny(isinstance(x0[t], stochasticDistribution) for t in s.Dep))
    cond2 = (isinstance(s, ooarray) and PythonAny(hasStochDep(t, x0) for t in s.view(ndarray)))
    return cond1 or cond2
#    (isinstance(s, ooarray) and PythonAny(hasStochDep(t) for t in s.view(ndarray)))
    
def Get(elem, p):
    elem2 = p.dictOfFixedFuncs.get(elem, None)
    if elem2 is not None:
        return elem2

    return p._dictOfRedirectedFuncs.get(elem, elem)

def formDictOfRedirectedFuncs(elem, p):
#    return
    from FuncDesigner.overloads import prod as fd_prod
    from FuncDesigner import mean as fd_mean, oofun, abs as fd_abs, std as fd_std, var as fd_var
    x0 = p._x0
    dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
#    if not isinstance(elem, oofun):# for more safety
#        return 
    #!!!!!!!!!!!! TODO: handle ooarray
    if not hasStochDep(elem, x0):
        return
    Iterator = elem.input if not elem._isSum else elem._summation_elements
    for elem_ in Iterator:
        elem_origin = elem_
        elem_ = dictOfRedirectedFuncs.get(elem_, elem_)
        if not isinstance(elem_, oofun):
            continue
        if elem_.engine in ('mean', 'std', 'var'):
#            print elem_.expr
            assert len(elem_.input) == 1, 'bug in FD kernel'
            inp = elem_.input[0]
#            if inp in p._dictOfRedirectedFuncs:
            if inp._isProd:
                inds = [True if hasStochDep(Get(s, p), x0) else False for s in inp._prod_elements]
                ordinaryElems = [Get(s, p) for j, s in enumerate(inp._prod_elements) if not inds[j]]
#                if len(ordinaryElems) == 0:
#                    continue
                stochElems = [Get(s, p) for j, s in enumerate(inp._prod_elements) if inds[j]]
                
                Tmp, tmp = fd_prod(ordinaryElems), fd_prod(stochElems)
#                formDictOfRedirectedFuncs(tmp, p)

                #TODO: rework _areFixed
                if isinstance(tmp, oofun) and p._areFixed(tmp.Dep):
                    tmp = tmp(p._x0)
                
                T = fd_mean(tmp) if elem_.engine == 'mean' else\
                fd_std(tmp) if elem_.engine == 'std' else \
                fd_var(tmp)
                
                if type(Tmp) == oofun:
                    formDictOfRedirectedFuncs(T, p)
                    T = dictOfRedirectedFuncs.get(T, T)
                    
                newElem = \
                Tmp * T if elem_.engine == 'mean' else \
                fd_abs(Tmp) * T if elem_.engine == 'std' else \
                Tmp**2 * T # for elem_.engine == 'var'
                
                dictOfRedirectedFuncs[elem_] = dictOfRedirectedFuncs[elem_origin] = newElem
                
                #TODO: store summation elements in dict
                # or mb store inputs in dict
                
                
                
#                elem_.input = [newElem]
        else:
            pass

# TODO: redirection for linear rest of sum
#def render_linear_parts(elem, p):
##    return
##    from FuncDesigner.overloads import prod as fd_prod
#    from FuncDesigner import oofun
#    x0 = p._x0
##    if not isinstance(elem, oofun):# for more safety
##        return 
#    #!!!!!!!!!!!! TODO: handle ooarray
#
#    Iterator = elem.input if not elem._isSum else elem._summation_elements
#    for elem_ in Iterator:
#        elem_origin = elem_
#        elem_ = p._dictOfRedirectedFuncs.get(elem_, elem_)
#        if not isinstance(elem_, oofun):
#            continue
#        if elem_.engine in ('mean', 'std', 'var'):
##            print elem_.expr
#            assert len(elem_.input) == 1, 'bug in FD kernel'
#            inp = elem_.input[0]
##            if inp in p._dictOfRedirectedFuncs:
#            if inp._isProd:
#                inds = [True if hasStochDep(s, x0) else False for s in inp._prod_elements]
#                ordinaryElems = [Get(s, p) for j, s in enumerate(inp._prod_elements) if not inds[j]]
#                if len(ordinaryElems) == 0:
#                    continue
#                stochElems = [Get(s, p) for j, s in enumerate(inp._prod_elements) if inds[j]]
#                
#                Tmp, tmp = fd_prod(ordinaryElems), fd_prod(stochElems)
#                if isinstance(tmp, oofun) and p._areFixed(tmp.Dep):
#                    tmp = tmp(p._x0)
#                newElem = \
#                Tmp * fd_mean(tmp) if elem_.engine == 'mean' else \
#                fd_abs(Tmp) * fd_std(tmp) if elem_.engine == 'std' else \
#                Tmp**2 * fd_var(tmp) # for elem_.engine == 'var'
#                
#                p._dictOfRedirectedFuncs[elem_] = p._dictOfRedirectedFuncs[elem_origin] = newElem
##                elem_.input = [newElem]
#        else:
#            pass



from baseProblem import MatrixProblem
from numpy import inf, dot, zeros, hstack, flatnonzero, ndarray, zeros_like
from ooMisc import norm
from oologfcn import OpenOptException
from nonOptMisc import scipyInstalled, Vstack, csr_matrix


class SLE(MatrixProblem):
    expectedArgs = ['C', 'd']# for FD it should be Cd and x0
    probType = 'SLE'
    goal = 'solution'
    allowedGoals = ['solution']
    showGoal = False
    FuncDesignerSign = 'C'
    solver = 'defaultSLEsolver'
    _optionalData = []
    _isPrepared = False
    
    def __init__(self, *args, **kwargs):
        MatrixProblem.__init__(self, *args, **kwargs)
    
    _useSparse = lambda self: True if (scipyInstalled and self.n > 100) else False

    def objFunc(self, x):
        if isinstance(self.C, ndarray):
            return norm(dot(self.C, x) - self.d, inf)
        else:
            # TODO: omit code clone in FD ooFun.py, function _D
            t1 = self.C_as_csc
            t2 = csr_matrix(x)
            if t2.shape[0] != t1.shape[1]:
                assert t2.shape[1] == t1.shape[1], 'incorrect shape in OpenOpt SLE, inform developers about the bug'
                t2 = t2.T
            rr =  t1._mul_sparse_matrix(t2)            
            return norm(rr.toarray().flatten() - self.d, inf)

    def _Prepare(self):
        if self._isPrepared: return
        self._isPrepared = True
        if isinstance(self.d, dict): # FuncDesigner startPoint 
            self.x0 = self.d
        MatrixProblem._Prepare(self)
        if self.isFDmodel:
            equations = self.C
            AsSparse = bool(self.useSparse) if type(self.useSparse) != str else self._useSparse()
            C, d = [], []
            if len(self._fixedVars) < len(self._freeVars):
                Z = dict([(v, zeros_like(self._x0[v]) if v not in self._fixedVars else self._x0[v]) for v in self._x0.keys()])
            else:
                Z = dict([(v, zeros_like(self._x0[v]) if v in self._freeVars else self._x0[v]) for v in self._x0.keys()])
            #Z = self.x0#self._vector2point(zeros(self.n))

            for lin_oofun in equations:
                if lin_oofun.getOrder(self.freeVars, self.fixedVars) > 1:
                    raise OpenOptException('SLE constructor requires all equations to be linear')
                C.append(self._pointDerivative2array(lin_oofun.D(Z, **self._D_kwargs), useSparse = AsSparse))
                d.append(-lin_oofun(Z))
                
            self.d = hstack(d).flatten()
            self.C  = Vstack(C)
            
            if hasattr(self.C, 'tocsc'): self.C_as_csc = self.C.tocsc()
            if isinstance(self.C,ndarray) and self.n > 100 and len(flatnonzero(self.C))/self.C.size < 0.3:
                s = "Probably you'd better solve this SLE as sparse"
                if not scipyInstalled: s += ' (requires scipy installed)'
                self.pWarn(s)


        self.x0 = zeros(self.C.shape[1])
#        if not self.damp is None and not any(isfinite(self.X)):
#            self.X = zeros(self.n)


#ff = lambda x, LLSPprob: LLSPprob.objFunc(x)
#def dff(x, LLSPprob):
#    r = dot(LLSPprob.C.T, dot(LLSPprob.C,x)  - LLSPprob.d)
#    if not LLSPprob.damp is None: r += LLSPprob.damp*(x - LLSPprob.X)
#    if LLSPprob.f is not None and all(isfinite(LLSPprob.f)) : r += LLSPprob.f
#    return r
#
#def d2ff(x, LLSPprob):
#    r = dot(LLSPprob.C.T, LLSPprob.C)
#    if not LLSPprob.damp is None: r += LLSPprob.damp*eye(x.size)
#    return r



''' LCPSolve(M,q): procedure to solve the linear complementarity problem:

       w = M z + q
       w and z >= 0
       w'z = 0

   The procedure takes the matrix M and vector q as arguments.  The
   procedure has three returns.  The first and second returns are
   the final values of the vectors w and z found by complementary
   pivoting.  The third return is a 2 by 1 vector.  Its first
   component is a 1 if the algorithm was successful, and a 2 if a
   ray termination resulted.  The second component is the value of
   the artificial variable upon termination of the algorithm.
   The third component is the number of iterations performed in the
   outer loop.
 
   Derived from: http://www1.american.edu/academic.depts/cas/econ/gaussres/optimize/quadprog.src
   (original GAUSS code by Rob Dittmar <dittmar@stls.frb.org> )

   Lemke's Complementary Pivot algorithm is used here. For a description, see:
   http://ioe.engin.umich.edu/people/fac/books/murty/linear_complementarity_webbook/kat2.pdf

Copyright (c) 2010 Rob Dittmar, Enzo Michelangeli and IT Vision Ltd

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

'''
   
from numpy import *

def LCPSolve(M,q, pivtol=1e-8): # pivtol = smallest allowable pivot element
    rayTerm = False
    loopcount = 0
    if (q >= 0.).all(): # Test missing in Rob Dittmar's code
        # As w - Mz = q, if q >= 0 then w = q and z = 0
        w = q
        z = zeros_like(q)
        retcode = 0.
    else:
        dimen = M.shape[0] # number of rows
        # Create initial tableau
        tableau = hstack([eye(dimen), -M, -ones((dimen, 1)), asarray(asmatrix(q).T)])
        # Let artificial variable enter the basis
        basis = list(range(dimen)) # basis contains a set of COLUMN indices in the tableau 
        locat = argmin(tableau[:,2*dimen+1]) # row of minimum element in column 2*dimen+1 (last of tableau)
        basis[locat] = 2*dimen # replace that choice with the row 
        cand = locat + dimen
        pivot = tableau[locat,:]/tableau[locat,2*dimen]
        tableau -= tableau[:,2*dimen:2*dimen+1]*pivot # from each column subtract the column 2*dimen, multiplied by pivot 
        tableau[locat,:] = pivot # set all elements of row locat to pivot
        # Perform complementary pivoting
        oldDivideErr = seterr(divide='ignore')['divide'] # suppress warnings or exceptions on zerodivide inside numpy
        while amax(basis) == 2*dimen:
            loopcount += 1
            eMs = tableau[:,cand]    # Note: eMs is a view, not a copy! Do not assign to it...
            missmask = eMs <= 0.
            quots = tableau[:,2*dimen+1] / eMs # sometimes eMs elements are zero, but we suppressed warnings...
            quots[missmask] = Inf # in any event, we set to +Inf elements of quots corresp. to eMs <= 0. 
            locat = argmin(quots)
            if abs(eMs[locat]) > pivtol and not missmask.all(): # and if at least one element is not missing 
                # reduce tableau
                pivot = tableau[locat,:]/tableau[locat,cand]
                tableau -= tableau[:,cand:cand+1]*pivot 
                tableau[locat,:] = pivot
                oldVar = basis[locat]
                # New variable enters the basis
                basis[locat] = cand
                # Select next candidate for entering the basis
                if oldVar >= dimen:
                    cand = oldVar - dimen
                else:
                    cand = oldVar + dimen
            else:
                rayTerm = True
                break
        seterr(divide=oldDivideErr) # restore original handling of zerodivide in Numpy
        # Return solution to LCP
        vars = zeros(2*dimen+1)
        vars[basis] = tableau[:,2*dimen+1]
        w = vars[:dimen]
        z = vars[dimen:2*dimen]    
        retcode = vars[2*dimen]
    # end if (q >= 0.).all() 
    
    if rayTerm:
        retcode = (2, retcode, loopcount)  # ray termination
    else:
        retcode = (1, retcode, loopcount)  # success
    return (w, z, retcode)




import random
import math
#import logging
import numpy as np
from openopt.kernel.setDefaultIterFuncs import IS_MAX_FUN_EVALS_REACHED

def P(prev_score,next_score,temperature):
    if next_score > prev_score:
        return 1.0
    else:
        return math.exp( -abs(next_score-prev_score)/temperature )

class ObjectiveFunction:
    '''class to wrap an objective function and 
    keep track of the best solution evaluated'''
    def __init__(self,objective_function):
        self.objective_function=objective_function
        self.best=None
        self.best_score=None
    
    def __call__(self,solution):
        score=self.objective_function(solution)
        if self.best is None or score > self.best_score:
            self.best_score=score
            self.best=solution
#            logging.info('new best score: %f',self.best_score)
        return score

def kirkpatrick_cooling(start_temp,alpha):
    T=start_temp
    while True:
        yield T
        T=alpha*T

def anneal(init_function,move_operator,objective_function,max_evaluations,start_temp,alpha, prob = None):
    
    # wrap the objective function (so we record the best)
    objective_function=ObjectiveFunction(objective_function)
    
    current=init_function()
    current_score=objective_function(current)
    num_evaluations=1
    
    cooling_schedule=kirkpatrick_cooling(start_temp,alpha)
    
#    logging.info('anneal started: score=%f',current_score)
    
    for temperature in cooling_schedule:
        done = False
        # examine moves around our current position
        for next in move_operator(current):
            if num_evaluations >= max_evaluations:
                if prob is not None:
                    prob.istop = IS_MAX_FUN_EVALS_REACHED
                    prob.msg = 'max objfunc evals limit (p.maxFunEvals=%d) has been reached' % prob.maxFunEvals
                done=True
                break
            
            next_score=objective_function(next)
            num_evaluations+=1
            
            # probablistically accept this solution
            # always accepting better solutions
            p=P(current_score,next_score,temperature)
            
            if not num_evaluations % 64 and prob is not None:
                prob.iterfcn(np.array(current), -current_score)
                if prob.istop != 0:
                    return (num_evaluations, objective_function.best_score, objective_function.best)            

            if random.random() < p:
                current=next
                current_score=next_score
                break
            
           
        # see if completely finished
        if done: break
    
    best_score=objective_function.best_score
    best=objective_function.best
#    logging.info('final temperature: %f',temperature)
#    logging.info('anneal finished: num_evaluations=%d, best_score=%f',num_evaluations,best_score)
    return (num_evaluations,best_score,best)



#!/usr/bin/python
'''
Simplest OpenOpt BPP (Bin Packing Problem) example;
requires FuncDesigner installed.
For some solvers limitations on time, cputime, "enough" value, basic GUI features are available.
See http://openopt.org/BPP for more details
'''
from openopt import *
from numpy import sin

N = 15 
# create N items: 
items = [{'volume': 2*sin(i) + 3} for i in range(N)] # i = 0, ..., N-1 (Python indexation starts from zero)
# bins (containers):
bins = {'volume': 13}
p = BPP(items, bins)
r = p.solve('glpk', iprint = 0) # requires cvxopt and glpk installed, see http://openopt.org/BPP for other solvers
'''
------------------------- OpenOpt 0.50 -------------------------
solver: glpk   problem: unnamed    type: MILP   goal: min
 iter   objFunVal   log10(maxResidual)   
    0  0.000e+00               0.00 
    1  4.000e+00            -100.00 
istop: 1000 (optimal)
Solver:   Time Elapsed = 0.3 	CPU Time Elapsed = 0.11
objFunValue: 4 (feasible, MaxResidual = 0)
'''
print(r.xf) # numbers of items in 1st, 2nd, 3rd, 4th bins
# [(2, 4, 9, 10), (0, 8, 14), (1, 3, 12), (5, 6, 7, 11, 13)]
# pay attention that Python indexation starts from zero: item 0, item 1 ...



"""
this is an example of using d2f - Hesse matrix (2nd derivatives)
d2c, d2h, d2l are intended to be implemented soon 
and to be connected to ALGENCAN and/or CVXOPT 
and/or other NLP solvers

//Dmitrey
"""
from openopt import NLP
from numpy import cos, arange, ones, asarray, abs, zeros, diag
N = 300
M = 5
ff = lambda x: ((x-M)**4).sum()
p = NLP(ff, cos(arange(N)))
p.df =  lambda x: 4*(x-M)**3
p.d2f = lambda x: diag(12*(x-M)**2)
# other valid assignment: 
# p = NLP(lambda x: ((x-M)**4).sum(), cos(arange(N)), df =  lambda x: 4*(x-M)**3, d2f = lambda x: diag(12*(x-M)**2))
# or 
# p = NLP(x0 = cos(arange(N)), f = lambda x: ((x-M)**4).sum(), df =  lambda x: 4*(x-M)**3, d2f = lambda x: diag(12*(x-M)**2))
r = p.solve('scipy_ncg')
print('objfunc val: %e' % r.ff) # it should be a small positive like 5.23656378549e-08




from openopt.kernel.baseSolver import baseSolver
import DIRECT, numpy as np
from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F

class direct(baseSolver):
    __name__ = 'direct'
    __license__ = "MIT"
    __authors__ = ''
    __alg__ = ""
    iterfcnConnected = False
    __homepage__ = ''

    __info__ = ""
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']
    __isIterPointAlwaysFeasible__ = True#lambda self, p: p.__isNoMoreThanBoxBounded__()
    _requiresFiniteBoxBounds = True
    
    
    eps=0.0001
#    maxf=20000
#    maxT=6000
    algmethod=0
#    fglobal=-1e+100
    fglper=0.01
    volper=-1.0
    sigmaper=-1.0
    logfilename='DIRresults.txt'
    
    funcForIterFcnConnection = 'f'
    _requiresBestPointDetection = True

    def __init__(self):pass
    def __solver__(self, p):

        #if not p.__isFiniteBoxBounded__(): p.err('this solver requires finite lb, ub: lb <= x <= ub')
        
        p.kernelIterFuncs.pop(SMALL_DELTA_X, None)
        p.kernelIterFuncs.pop(SMALL_DELTA_F, None)

        def objective(x, userData = None):
            r = p.f(x)
            has_nan = int(np.any(np.isnan(r)))
            return r, has_nan

        lb, ub = p.lb, p.ub
#        lb[lb < -1e20] = -1e20
#        ub[ub > 1e20] = 1e20

        maxf = min((p.maxFunEvals, 19999))
        maxT = min((p.maxIter, 89999))
        fglobal = max((p.fOpt, -1e+100))
        
        xf, fmin, ierror = DIRECT.solve(objective, lb, ub, eps=self.eps, maxf=maxf, maxT=maxT, 
                                        algmethod=self.algmethod, fglobal=fglobal, fglper=self.fglper, 
                                        volper=self.volper, sigmaper=self.sigmaper, 
                                        logfilename=self.logfilename)
        
        p.xf, p.ff = xf, fmin
        if p.istop == 0: 
            p.istop = 1000
            p.msg = ierror




from numpy import eye
def pointProjection(x,  lb, ub, A, b, Aeq, beq):
    from openopt import QP
    # projection of x to set of linear constraints
    n = x.size
    # TODO: INVOLVE SPARSE CVXOPT MATRICES
    p = QP(H = eye(n), f = -x, A = A, b=b, Aeq=Aeq, beq=beq, lb=lb, ub=ub)
    #r = p.solve('cvxopt_qp', iprint = -1)
    r = p.solve('nlp:scipy_slsqp', contol = 1e-8, iprint = -1)
    return r.xf



from NLOPT_AUX import NLOPT_AUX
from NLOPT_BASE import NLOPT_BASE
import nlopt

class bobyqa(NLOPT_BASE):
    __name__ = 'bobyqa'
    __alg__ = 'Bound constrained Optimization BY Quadratic Approximation'
    __authors__ = 'Michael JD Powell'
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']
    __isIterPointAlwaysFeasible__ = True
    
    funcForIterFcnConnection = 'f'
    
    def __solver__(self, p):
#        if p.n < 15:
#            p.f_iter = 4
#        else:
#            p.f_iter = p.n/4

        
        NLOPT_AUX(p, nlopt.LN_BOBYQA)



from openopt.kernel.baseSolver import baseSolver
from mlabwrap import mlab

scipy_installed = False
try:
    from scipy.sparse import find, isspmatrix
    scipy_installed = True
except ImportError:
    pass

class MATLAB_BASE(baseSolver):
    __license__ = "proprietary"
    __authors__ = "Mathworks"
    _requiresBestPointDetection = True
    
    def _Solve(self, p, *args):
        SolverName = self.__name__
        if scipy_installed:
            Args = [arg if not isspmatrix(arg) else scipy_sparse_to_mlab_sparse(arg) for arg in args]
        else:
            Args = list(args)
        
        opts = []
        for fn in self.opts: 
            opts += [fn, optConverter(p, fn)]
        opts = mlab.optimoptions(SolverName, *opts)
        
        Args.append(opts)
        r = getattr(mlab, SolverName)(*Args)
        p.istop = 1000
        return r

def optConverter(p, fn):
    if fn == 'Display':
        return 'off' if p.iprint < 0 else 'final' if p.iprint == 0 else 'iter'
    elif fn == 'TolFun':
        return p.ftol
    elif fn == 'TolX':
        return p.xtol
    elif fn == 'MaxIter':
        return p.maxIter
    elif fn == 'MaxTime':
        return p.maxTime
    elif fn == 'TolXInteger':
        return p.discrtol
    else:
        assert 0, 'incorrect or unimplemented in OpenOpt yet MATLAB param'
        
def scipy_sparse_to_mlab_sparse(scipy_sparse_matrix): 
    I, J, values = find(scipy_sparse_matrix)
    return mlab.sparse(I+1, J+1, values, scipy_sparse_matrix.shape[0], scipy_sparse_matrix.shape[1]) 



#PythonSum = sum
import NLP

from ooMisc import isspmatrix
from baseProblem import MatrixProblem
from numpy import asfarray, dot, nan, zeros, isfinite, all, ravel, diag, isscalar, arange, ndarray
from copy import copy as PythonCopy


class QP(MatrixProblem):
    probType = 'QP'
    goal = 'minimum'
    allowedGoals = ['minimum', 'min']
    showGoal = False
    _optionalData = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'QC', 'intVars']
    expectedArgs = ['H', 'f']
    FuncDesignerSign = 'H'
    C = 0.0
    
    def _Prepare(self):
        if not self._isFDmodel():
            if not isspmatrix(self.H):
                self.H = asfarray(self.H) # TODO: handle the case in runProbSolver()
            self.n = self.H.shape[0]
            if not hasattr(self, 'x0') or self.x0 is None or self.x0[0] is nan:
                self.x0 = zeros(self.n)
        MatrixProblem._Prepare(self)
        if self._isFDmodel():
            H, f, C = quad_render(self.H, self)
            self.user.f = (self.H, )
            self.H, self.f, self.C = H, f, C
#            print(H, f, C)
            
            if self.fixedVars is None or (self.freeVars is not None and len(self.freeVars)<len(self.fixedVars)):
                order_kw = {'Vars': self.freeVarsSet}
            else:
                order_kw = {'fixedVars': self.fixedVarsSet}
            order_kw['fixedVarsScheduleID'] = self._FDVarsID 
            from FuncDesigner import ooarray
            for c in self.constraints:
                if isinstance(c, ooarray):
                    for elem in c:
                        processConstraint(elem, order_kw, self)
                else:
                    processConstraint(c, order_kw, self)
            r = []
            for v in self._freeVarsList:
                if isinstance(v.domain, (tuple, list, ndarray, set)):
                    self.err('for FuncDesigner MILP models only variables with domains int, bool or None (real) are implemented for now')
                if v.domain is int or v.domain is  'int' or v.domain is bool or v.domain is 'bool':
                    r1, r2 = self._oovarsIndDict[v]
                    r += arange(r1, r2).tolist()
            
            self.intVars, self._intVars = r, self.intVars
        self._intVars_vector = self.intVars
    
    def __init__(self, *args, **kwargs):
        self.QC = []
        MatrixProblem.__init__(self, *args, **kwargs)
        if self._isFDmodel():
            if len(args) > 1:
                self.x0 = args[1]
            self.f = args[0]
        else:
            if len(args) > 1 or 'f' in kwargs.keys():
                self.f = ravel(self.f)

    def objFunc(self, x):
        return asfarray(0.5*dot(x, self.matMultVec(self.H, x)) + dot(self.f, x).sum()).flatten() + self.C

    def qp2nlp(self, solver, **solver_params):
        if hasattr(self,'x0'): p = NLP.NLP(ff, self.x0, df=dff, d2f=d2ff)
        else: p = NLP.NLP(ff, zeros(self.n), df=dff, d2f=d2ff)
        p.args.f = self # DO NOT USE p.args = self IN PROB ASSIGNMENT!
        p.iprint = self.iprint
        self.fill(p)
        self.iprint = -1
        

        # for QP plot is via NLP
        p.show = self.show
        p.plot, self.plot = self.plot, 0

        #p.checkdf()
        r = p.solve(solver, **solver_params)
        self.xf, self.ff, self.rf = r.xf, r.ff, r.rf
        return r

    def __finalize__(self):
        if self.isFDmodel: self.intVars = self._intVars
        MatrixProblem.__finalize__(self)

ff = lambda x, QProb: QProb.objFunc(x)
def dff(x, QProb):
    r = dot(QProb.H, x)
    if all(isfinite(QProb.f)) : r += QProb.f
    return r

def d2ff(x, QProb):
    r = QProb.H
    return r

def processConstraint(c, order_kw, p):
    if c is True:
        return
    order = c.oofun.getOrder(**order_kw)
    if order < 2:
        return
    elif order > 2:
        assert 0, 'at most quadratic constraint is expected'
    H, f, C = quad_render(c.oofun, p)
    finite_lb = isfinite(c.lb)
    finite_ub = isfinite(c.ub)
    assert np.logical_xor(finite_lb, finite_ub), 'exactly one finite bound is expected'
    if finite_lb:
        p.QC.append((-H, -f, -c.lb - C))
    else:#finite_ub
        p.QC.append((H, f, -c.ub + C))
        

from nonOptMisc import scipyInstalled
import numpy as np

def quad_render(arg, p):
#    from FuncDesigner import oofun
    from FuncDesigner.ooPoint import ooPoint
    from FuncDesigner.baseClasses import OOArray
    if isinstance(arg, OOArray):
        assert arg.size == 1, 'quad_render works with oofuns with scalar output only'
        arg = arg.item()
    #assert type(arg) == oofun, 'oofun input expected'
    assert p.fixedVars is None, 'quadratic rendering is unimplemented for fixedVars yet'
    
    optVarSizes = p._optVarSizes
#    print(optVarSizes.values(), np.array(optVarSizes.values()))
    assert all(np.array(list(optVarSizes.values())) == 1), 'quadratic rendering is unimplemented for oovar(size=n) yet, use oovars(n) instead'
        
    oovarsIndDict = p._oovarsIndDict
    n = p.n
    
    if 1 and scipyInstalled and p.useSparse is not False:
        useSparse = p.useSparse
        if useSparse == 'auto':
            useSparse = p.n > 150
    else:
        useSparse = False
        
    if useSparse:
        from scipy.sparse import lil_matrix
        H = lil_matrix((n, n))
    else:
        H = np.zeros((n, n))
    f = np.zeros(n)
    c = 0.0
    
    startPoint = p._x0
    
    if p.fixedVars is None or (p.freeVars is not None and len(p.freeVars)<len(p.fixedVars)):
        order_kw = {'Vars': p.freeVarsSet}
        Z = dict((v, np.zeros_like(p._x0[v]) if v in p._freeVars else p._x0[v]) for v in p._x0.keys())
    else:
        order_kw = {'fixedVars': p.fixedVarsSet}
        Z = dict((v, np.zeros_like(p._x0[v]) if v not in p._fixedVars else p._x0[v]) for v in p._x0.keys())
    Z = ooPoint(Z)
    D_kwargs = p._D_kwargs
    D_kwargs['useSparse'] = useSparse
    order_kw['fixedVarsScheduleID'] = p._FDVarsID 
    
#    D = arg.D(Z, **D_kwargs)
    
    elems = PythonCopy(arg._summation_elements) if arg._isSum else [arg]
    
    while(len(elems) != 0):
        elem = elems.pop()
        if np.isscalar(elem) or type(elem) == np.ndarray:
            c += elem
            continue
        order = elem.getOrder(**order_kw)
        assert order <= 2, 'constant,linear or quadratic function is expected'
        if order == 0:
            c += elem(startPoint)
        elif order == 1:
            # TODO: mb rework fd.sum() to sum(linear) + sum(nonlin)
            pointDerivative = elem.D(Z, **D_kwargs)
            if len(pointDerivative) != 0:
                # TODO: check iadd with different types (dense/sparse)
                asdf(p, pointDerivative, useSparse, f)
#                tmp = p._pointDerivative2array(pointDerivative, useSparse = useSparse)
#                if hasattr(tmp, 'toarray'):
#                    tmp = tmp.toarray()
#                f += tmp.flatten()
            c += elem(Z)
        elif order == 2:
            if elem._isProd:
                prod_elements =  elem._prod_elements
                last_prod_elem = prod_elements[-1]
                lastElemIsScalar = np.isscalar(last_prod_elem) or\
                    (type(last_prod_elem) == np.ndarray and last_prod_elem.size == 1)
                L = len(prod_elements)
                if L > 2:
                    assert lastElemIsScalar, 'unimplemented yet'
                    assert len(prod_elements) == 3
                    elem1, elem2, koeff = prod_elements
                    squared_elem = None
                else:# L == 2:
                    if lastElemIsScalar:
                        koeff = last_prod_elem
                        tmp = prod_elements[0]
                        if tmp._isSum:
                            elems += [koeff * Elem for Elem in tmp._summation_elements]
                            continue
                        else:
                            squared_elem = tmp.input[0] if not tmp.is_oovar else tmp
                        elem1 = elem2 = None
                    else:
                        koeff = None
                        elem1, elem2 = prod_elements
                        squared_elem = None
                if elem2 is not None and elem2._isSum:
                    elem1, elem2 = elem2, elem1
                if elem1 is not None and elem1._isSum:
                    Tmp = koeff*elem2 if koeff is not None else elem2
                    elems += [Tmp * Elem for Elem in elem1._summation_elements]
                    continue
            
            else:
                if elem.fun == np.sum: # oofun.sum()
                    assert len(elem.input) == 1
                    elems.append(elem.input[0])
                    continue
                else:
                    squared_elem = elem.input[0] if not elem.is_oovar else elem
                koeff = None
            if squared_elem is not None:
#                tmp = squared_elem.input[0] if not squared_elem.is_oovar else squared_elem
#                assert tmp.is_oovar, 'unimplemented yet'
#                d = tmp.D(Z, **D_kwargs)
                d = squared_elem.D(Z, **D_kwargs)

                #assert len(d) == 1, 'unimplemented yet'
                if len(d) > 1:
                    assert squared_elem._isSum, 'unimplemented yet'
                    Tmp = squared_elem * koeff if koeff is not None else squared_elem
                    elems += [Tmp * Elem for Elem in squared_elem._summation_elements]
                    continue
                oov, lin_coeff = list(d.items())[0]
                const = squared_elem(Z)
#                print('const:', const)
#                const = tmp(Z)
                if isscalar(lin_coeff):
                    Tmp2 = np.float(lin_coeff ** 2)  
                    Tmp1 = 2.0 * lin_coeff * const
                else:
                    Diag = diag(lin_coeff)
                    Tmp2 = Diag **2.0
                    Tmp1 = 2.0 * Diag * const
                Tmp0 = const ** 2.0
                if koeff is not None:
                    Tmp2 *= koeff
                    Tmp1 *= koeff
                    Tmp0 *= koeff
                Ind = oovarsIndDict[oov]
#                assert Ind[1]-Ind[0] == 1, 'unimplemented yet'
                if Ind[1]-Ind[0] == 1:
                    H[Ind[0], Ind[0]] += Tmp2
                    f[Ind[0]] += Tmp1
                else:
                    H[range(Ind[0], Ind[1]), range(Ind[0], Ind[1])] += Tmp2
                    f[range(Ind[0], Ind[1])] += Tmp1
                c += Tmp0 if isscalar(Tmp0) else Tmp0.sum()
#                for k, v in d.items():
#                    pass
            else:
                assert elem1.is_oovar and elem2.is_oovar, 'unimplemented yet'
                Tmp = 1.0 if koeff is None else koeff
                Ind1, Ind2 = oovarsIndDict[elem1], oovarsIndDict[elem2]
                assert Ind1[1]-Ind1[0] == 1, 'unimplemented yet'
                assert Ind2[1]-Ind2[0] == 1, 'unimplemented yet'
                
                H[Ind1[0], Ind2[0]] += Tmp
                
    H = H + H.T
    return H, f, c

#def updateH(H, ind1, ind2, elem):
#    H[ind1, ind2] += elem

def asdf(p, pointDerivative, useSparse, f):
    if 1 or useSparse is False:
        for k, v in pointDerivative.items():
            Ind = p._oovarsIndDict[k]
            if type(v) in (float, np.float64):
                f[Ind[0]]+=v
            else:
                f[Ind[0]:Ind[1]] += v if type(v) != ndarray else v.item()
    else:
        tmp = p._pointDerivative2array(pointDerivative, useSparse = useSparse)
        if hasattr(tmp, 'toarray'):
            tmp = tmp.toarray()
        f += tmp.flatten()

    
    
#    if p.fixedVars is None or (p.freeVars is not None and len(p.freeVars)<len(p.fixedVars)):
#        Z = dict([(v, zeros_like(self._x0[v]) if v in self._freeVars else self._x0[v]) for v in self._x0.keys()])
##        varIsFixed = lambda v: v not in p.freeVars
#    else:
#        Z = dict([(v, zeros_like(self._x0[v]) if v not in self._fixedVars else self._x0[v]) for v in self._x0.keys()])
##        varIsFixed = lambda v: v in p.fixedVars
    
#    Elems = arg._summation_elements if arg._isSum else [arg]
#    for elem in Elems:
#        if isinstance(elem, OOArray):
#            assert elem.size == 1, 'quadratic rendering is implemented for oofuns with scalar output only'
#            elem = elem.item()
#            
#        if isinstance(elem, oofun):
#            if elem.is_oovar:
#                if varIsFixed(elem):
#                    c += startPoint[elem]
#                f[oovarsIndDict[elem]] += 1.0
#            else:
#                if elem._alt__neg__ is not None:
#                    elem = (-1)*elem._neg_elem
#                assert elem._isProd, 'incompatible oofun or bug in quad_render'
#                elem1, elem2 =  elem._prod_elements[0], elem._prod_elements[-1]
#                assert isinstance(elem1, oofun)
#                if isinstance(elem2, oofun):
#                    
#                
#                
#        elif isinstance(elem, np.ndarray):
#            assert elem.size == 1, 'quadratic rendering is implemented for oofuns with scalar output only'
#            c += elem
#        else:
#            assert 0, 'quadratic rendering is implemented for type %s' % type(elem)



from baseProblem import MatrixProblem
from numpy import asarray, ones, inf, dot, zeros, asfarray, atleast_1d
import NLP

class LP(MatrixProblem):
    _optionalData = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub']
    expectedArgs = ['f', 'x0']
    probType = 'LP'
    allowedGoals = ['minimum', 'min', 'max', 'maximum']
    showGoal = True
    _lp_prepared = False

    def __init__(self, *args, **kwargs):
        self.goal = 'minimum'
        MatrixProblem.__init__(self, *args, **kwargs)
        if len(args) > 1 and not hasattr(args[0], 'is_oovar'):
            self.err('No more than 1 argument is allowed for classic style LP constructor')

    def _Prepare(self):
        if self._lp_prepared: return
        self._lp_prepared = True
        MatrixProblem._Prepare(self)
        if self.x0 is None: self.x0 = zeros(self.n)
        if hasattr(self.f, 'is_oovar'): # hence is oofun or oovar
            # freeVars and fixedVars are already rendered to both not-None here (if it's the case)
            # but current .D() implementation doesn't allow to use both arguments
            # BTW translator ignores fixed vars in derivatives, so passing fixedVars here can be omitted 
            # but is performed for more safety wrt future changes
            _f = self._point2vector(self.f.D(self._x0, fixedVars = self.fixedVars)) 
            self.f, self._f = _f, self.f
            self.user.f = (self._f, )
            self._init_f_vector = _f # we don't take p.goal into account here
            _c = self._f(self._x0) - dot(self.f, self.x0)
            self._c = _c
        else:
            self._init_f_vector = self.f # we don't take p.goal into account here
            self._c = 0
        self.f = atleast_1d(self.f)
        if not hasattr(self, 'n'): self.n = len(self.f)
        #print 'lb:', self.lb, 'ub:', self.ub
        if not hasattr(self, 'lb'): self.lb = -inf * ones(self.n)
        if not hasattr(self, 'ub'): self.ub = inf * ones(self.n)
#        if any(isnan(self.lb)): 
#            if self.lb.size != 1: self.err('NaN in lower bound for a variable from the problem')
#            self.lb = -inf * ones(self.n)
#        if any(isnan(self.ub)): 
#            if self.ub.size != 1: self.err('NaN in upper bound for a variable from the problem')
#            self.ub = inf * ones(self.n)
        
        if self.goal in ['max', 'maximum']:
            self.f = -asfarray(self.f)
            
    # TODO: handle this and SDP finalize in single func finalize_for_max
    def __finalize__(self):
        MatrixProblem.__finalize__(self)
        if self.goal in ['max', 'maximum']:
            self.f = -self.f
            for fn in ['fk', ]:#not ff - it's handled in other place in RunProbSolver.py
                if hasattr(self, fn):
                    setattr(self, fn, -getattr(self, fn))
        if hasattr(self, '_f'):
            self.f = self._f
            

            
    def objFunc(self, x):
        if self.isFDmodel:
            # TODO: fix it
            r = self._f(self._vector2point(x))
            return -r if self.goal in ['max', 'maximum'] else r
        else:
            return dot(self.f, x) + self._c

    def lp2nlp(self, solver, **solver_params):
        if self.isConverterInvolved and self.goal in ['max', 'maximum']:
            self.err('maximization problems are not implemented lp2nlp converter')
        ff = lambda x: dot(x, self.f)+self._c
        dff = lambda x: self.f
        if hasattr(self,'x0'): p = NLP.NLP(ff, self.x0, df=dff)
        else: p = NLP.NLP(ff, zeros(self.n), df=dff)
        self.fill(p)
        self.iprint = -1

        # for LP plot is via NLP
        p.show = self.show
        p.plot, self.plot = self.plot, 0
        if self.isFDmodel:
            p._x0 = self._x0 # may be used in interalg 
        r = p.solve(solver, **solver_params)
        self.xf, self.ff, self.rf = r.xf, r.ff, r.rf

        return r

    def exportToMPS(self, filename, format='fixed', startIndex=0):
        try: from lp_solve import lpsolve
        except ImportError: self.err('To export LP/MILP in files you should have lpsolve and its Python binding properly installed')
        
        maxNameLength = 8 if format != 'free' else 255
        handler = self.get_lpsolve_handler(maxNameLength, startIndex)
        
        # TODO: uncomment it
        ext = 'mps' if not filename.endswith('MPS') and not filename.endswith('mps') else ''
        if ext != '': filename += '.' + ext
        
        if format=='fixed':
            r = bool(lpsolve('write_mps', handler, filename) )
        elif format=='free':
            r = bool(lpsolve('write_freemps', handler, filename) )
        else:
            self.err('incorrect MPS format, should be "fixed" or "free"')
        if r != True: 
            self.warn('Failed to write MPS file, maybe read-only filesystem, incorrect path or write access is absent')
            
        lpsolve('delete_lp', handler) 
        return r

    def get_lpsolve_handler(self, maxNameLength=255, startIndex=0):
        try: from lp_maker import lp_maker, lpsolve
        except ImportError: self.err('To export LP/MILP in files you should have lpsolve and its Python binding properly installed')
        self._Prepare()
        from ooMisc import LinConst2WholeRepr
        LinConst2WholeRepr(self)
        
        # set goal to  min/max
        minim = 0 if self.goal in ['max', 'maximum'] else 1
        
        # objective
        f = self._init_f_vector
        
        lp_handle = lp_maker(List(asarray(f).flatten()), List(self.Awhole), List(asarray(self.bwhole).flatten()), List(asarray(self.dwhole).flatten()), \
        List(self.lb), List(self.ub), (1+asarray(self._intVars_vector)).tolist(), 0,minim)
       
        #lp_handle = lpsolve('make_lp', len(self.beq)+len(self.b), self.n) 
        L = lambda action, *args: lpsolve(action, lp_handle, *args)
        
        # set name
        L('set_lp_name', self.name)
        
        # set boolean values if present
        #if len(self.boolVars)>0:
            #assert self.boolVars[0] in [True, False]
            #L('set_binary', self.boolVars)
        
        # set variables names
        if self.isFDmodel:
            assert not isinstance(self.freeVars, set), 'error in openopt kernel, inform developers'
            #assert len(self.freeVars) == self.n, 'not implemented yet for oovars of size > 1'
            #names = [oov.name for oov in self.freeVars]
            x0 = self._x0
            names = []
            #assert not isinstance(self.freeVars,  set), 'error in openopt kernel, inform developers'
            for oov in self.freeVars:
                if oov.name.startswith('unnamed'):
                    L('delete_lp')
                    self.err('For exporting FuncDesigner models into MPS files you cannot have variables with names starting with "unnamed"')
                if ' ' in oov.name:
                    L('delete_lp')
                    self.err('For exporting FuncDesigner models into MPS files you cannot have variables with spaces in names')
                Size = asarray(x0[oov]).size
                if Size == 1:
                    Name = oov.name
                    names.append(Name)
                else:
                    tmp = [(oov.name + ('_%d' % (startIndex+j))) for j in range(Size)]
                    names += tmp
                    Name = tmp[-1]
                if maxNameLength < len(Name):
                    L('delete_lp')
                    self.err('incorrect name "%s" - for exporting FuncDesigner models into MPS files you cannot have variables with names of length > maxNameLength=%d'% maxNameLength)
                    
            # TODO: check are names unique
            L('set_col_name', names) 
        return lp_handle
        
def List(x):
    if isinstance(x, list): return x
    elif x == None or x.size == 0: return None
    else: return x.tolist()        




#!/usr/bin/python
'''
Simplest OpenOpt TSP example;
requires networkx (http://networkx.lanl.gov)
and FuncDesigner installed.
For some solvers limitations on time, cputime, "enough" value, basic GUI features are available.
See http://openopt.org/TSP for more details
'''
from openopt import *
from numpy import sin, cos
import networkx as nx

N = 5

G = nx.Graph()
G.add_edges_from(\
                 [(i,j,{'time': 1.5*(cos(i)+sin(j)+1)**2, 'cost':(i-j)**2 + 2*sin(i) + 2*cos(j)+1}) for i in range(N) for j in range(N) if i != j ])

# default objective is "weight"
# parameter "start" (node identifier, number or string) is optional
p = TSP(G, objective = 'time', start = 2) #, [optional] returnToStart={True}|False, constraints = ..., etc

r = p.solve('glpk') # also you can use some other solvers - sa, interalg, OpenOpt MILP solvers

# if your solver is cplex or interalg, you can provide some stop criterion, 
# e.g. maxTime, maxCPUTime, fEnough etc, for example 
# r = p.solve('cplex', maxTime = 100, fEnough = 10) 
# i.e. stop if solution with 10 nodes has been obtained or 100 sec were elapsed
# (gplk and lpSolve has no iterfcn connected and thus cannot handle those criterions)

# also you can use p.manage() to enable basic GUI (http://openopt.org/OOFrameworkDoc#Solving) 
# it requires tkinter installed, that is included into PythonXY, EPD;
# for linux use [sudo] easy_install tk or [sodo] apt-get install python-tk
#r = p.manage('cplex')

print(r.nodes)
print(r.edges)
print(r.Edges)
'''
Solver:   Time Elapsed = 0.05 	CPU Time Elapsed = 0.04 
# (glpk takes 5 sec for N = 15, Intel Atom 1.6 GHz)
objFunValue: 5.1628774 (feasible, MaxResidual = 0)
[2, 0, 4, 1, 3, 2]
[(2, 0), (0, 4), (4, 1), (1, 3), (3, 2)]
[(2, 0, {'cost': 8.8185948536513639, 'time': 0.51132677471086396}), 
(0, 4, {'cost': 17.486395009384143, 'time': 0.17994411205270408}), 
(4, 1, {'cost': 9.5669996211204236, 'time': 2.1164007698022411}), 
(1, 3, {'cost': 6.3628446278560142, 'time': 1.0875234238200695}), 
(3, 2, {'cost': 1.4499463430254496, 'time': 1.26768233210464})]
'''
